import decimal
import re

from medical.models import Item

from api_fhir_r4.configurations import GeneralConfiguration, R4IdentifierConfig
from api_fhir_r4.converters import MedicationConverter
from api_fhir_r4.mapping.medicationMapping import ItemVenueTypeMapping
from api_fhir_r4.mapping.patientMapping import PatientCategoryMapping
from api_fhir_r4.tests import GenericTestMixin
from fhir.resources.R4B.codeableconcept import CodeableConcept
from fhir.resources.R4B.extension import Extension
from fhir.resources.R4B.identifier import Identifier
from fhir.resources.R4B.money import Money
from fhir.resources.R4B.medication import Medication
from fhir.resources.R4B.ratio import Ratio
from fhir.resources.R4B.timing import Timing
from fhir.resources.R4B.quantity import Quantity
from fhir.resources.R4B.usagecontext import UsageContext


class MedicationTestMixin(GenericTestMixin):

    _TEST_MEDICATION_CODE = "TEST1"
    _TEST_MEDICATION_NAME = "TEST TABS 300MG"
    _TEST_MEDICATION_TYPE = "D"
    _TEST_MEDICATION_PACKAGE = "1000TABLETS"
    _TEST_MEDICATION_PRICE = 5.99
    _TEST_MEDICATION_CARE_TYPE = "B"
    _TEST_MEDICATION_FREQUENCY = 3
    _TEST_MEDICATION_PATIENT_CATEGORY = 15

    def create_test_imis_instance(self):
        imis_item = Item()
        imis_item.code = self._TEST_MEDICATION_CODE
        imis_item.name = self._TEST_MEDICATION_NAME
        imis_item.type = self._TEST_MEDICATION_TYPE
        imis_item.package = self._TEST_MEDICATION_PACKAGE
        imis_item.price = self._TEST_MEDICATION_PRICE
        imis_item.care_type = self._TEST_MEDICATION_CARE_TYPE
        imis_item.frequency = self._TEST_MEDICATION_FREQUENCY
        imis_item.patient_category = self._TEST_MEDICATION_PATIENT_CATEGORY
        return imis_item

    def verify_imis_instance(self, imis_obj):
        self.assertEqual(self._TEST_MEDICATION_CODE, imis_obj.code)
        self.assertEqual(self._TEST_MEDICATION_NAME, imis_obj.name)
        self.assertEqual(self._TEST_MEDICATION_TYPE, imis_obj.type)
        self.assertEqual(self._TEST_MEDICATION_PACKAGE, imis_obj.package)
        self.assertAlmostEqual(decimal.Decimal(self._TEST_MEDICATION_PRICE), imis_obj.price, places=2)
        self.assertEqual(self._TEST_MEDICATION_CARE_TYPE, imis_obj.care_type)
        self.assertEqual(self._TEST_MEDICATION_FREQUENCY, imis_obj.frequency)
        self.assertEqual(self._TEST_MEDICATION_PATIENT_CATEGORY, imis_obj.patient_category)

    def create_test_fhir_instance(self):
        fhir_medication = Medication.construct()
        code = MedicationConverter.build_fhir_identifier(
            self._TEST_MEDICATION_CODE,
            R4IdentifierConfig.get_fhir_identifier_type_system(),
            R4IdentifierConfig.get_fhir_generic_type_code()
        )
        identifiers = [code]
        fhir_medication.identifier = identifiers

        fhir_medication.form = MedicationConverter.build_codeable_concept("package", text=self._TEST_MEDICATION_PACKAGE)
        amount = MedicationConverter.split_package_amount(self._TEST_MEDICATION_PACKAGE)
        ratio = Ratio.construct()
        numerator = Quantity.construct()
        numerator.value = amount
        ratio.numerator = numerator
        fhir_medication.amount = ratio

        unit_price = MedicationConverter.build_fhir_unit_price_extension(self._TEST_MEDICATION_PRICE)
        fhir_medication.extension = [unit_price]

        medication_type = MedicationConverter.build_fhir_medication_type_extension(self._TEST_MEDICATION_TYPE)
        fhir_medication.extension.append(medication_type)

        medication_frequency = MedicationConverter.build_fhir_medication_frequency_extension(self._TEST_MEDICATION_FREQUENCY)
        fhir_medication.extension.append(medication_frequency)

        extension_usage = Extension.construct()
        extension_usage.url = f"{GeneralConfiguration.get_system_base_url()}StructureDefinition/medication-usage-context"

        # gender
        PatientCategoryMapping.load()
        male_flag = PatientCategoryMapping.imis_patient_category_flags["male"]
        female_flag = PatientCategoryMapping.imis_patient_category_flags["female"]
        extension = Extension.construct()
        extension.url = "Gender"
        administrative_system = "http://hl7.org/fhir/administrative-gender"
        extension.valueUsageContext = UsageContext.construct()
        extension.valueUsageContext.valueCodeableConcept = CodeableConcept.construct()
        if self._TEST_MEDICATION_PATIENT_CATEGORY & male_flag:
            coding_male = MedicationConverter._build_fhir_coding(code="male", display="Male", system=administrative_system)
            MedicationConverter._append_to_list_codeable_concept(extension, coding_male)
        if self._TEST_MEDICATION_PATIENT_CATEGORY & female_flag:
            coding_female = MedicationConverter._build_fhir_coding(code="female", display="Female", system=administrative_system)
            MedicationConverter._append_to_list_codeable_concept(extension, coding_female)
        system_gender = "http://terminology.hl7.org/CodeSystem/usage-context-type"
        extension.valueUsageContext.code = MedicationConverter._build_fhir_coding(code="gender", display="Gender", system=system_gender)
        gender = extension
        extension_usage.extension = [gender]

        # age type
        adult_flag = PatientCategoryMapping.imis_patient_category_flags["adult"]
        child_flag = PatientCategoryMapping.imis_patient_category_flags["child"]
        extension = Extension.construct()
        extension.url = "Age"
        usage_context_system = "http://terminology.hl7.org/CodeSystem/usage-context-type"
        age_type_system = f"{GeneralConfiguration.get_system_base_url()}CodeSystem/usage-context-age-type"
        extension.valueUsageContext = UsageContext.construct()
        extension.valueUsageContext.valueCodeableConcept = CodeableConcept.construct()
        if self._TEST_MEDICATION_PATIENT_CATEGORY & adult_flag:
            coding_adult = MedicationConverter._build_fhir_coding(code="adult", display="Adult", system=age_type_system)
            MedicationConverter._append_to_list_codeable_concept(extension, coding_adult)
        if self._TEST_MEDICATION_PATIENT_CATEGORY & child_flag:
            coding_child = MedicationConverter._build_fhir_coding(code="child", display="Child", system=age_type_system)
            MedicationConverter._append_to_list_codeable_concept(extension, coding_child)
        extension.valueUsageContext.code = MedicationConverter._build_fhir_coding(code="age", display="Age", system=usage_context_system)

        age = extension
        extension_usage.extension.append(age)

        code = ""
        if self._TEST_MEDICATION_CARE_TYPE == "O":
            code = "AMB"
        if self._TEST_MEDICATION_CARE_TYPE == "I":
            code = "IMP"
        if self._TEST_MEDICATION_CARE_TYPE == "B":
            code = "B"
        display = ItemVenueTypeMapping.item_venue_type[code]
        extension = Extension.construct()
        usage_context_system = "http://terminology.hl7.org/CodeSystem/usage-context-type"
        venue_system = "http://terminology.hl7.org/CodeSystem/v3-ActCode"
        if self._TEST_MEDICATION_CARE_TYPE is not None:
            if code != "B":
                extension.url = "CareType"
                extension.valueUsageContext = UsageContext.construct()
                extension.valueUsageContext.valueCodeableConcept = CodeableConcept.construct()
                coding_venue = MedicationConverter._build_fhir_coding(code=code, display=display, system=venue_system)
                MedicationConverter._append_to_list_codeable_concept(extension, coding_venue)
            else:
                MedicationConverter.build_fhir_both_care_type(extension, venue_system)
            extension.valueUsageContext.code = MedicationConverter._build_fhir_coding(
                code="venue", display="Clinical Venue", system=usage_context_system)

        care_type = extension
        extension_usage.extension.append(care_type)
        fhir_medication.extension.append(extension_usage)

        fhir_medication.code = MedicationConverter.build_codeable_concept(self._TEST_MEDICATION_CODE, text=self._TEST_MEDICATION_NAME)
        fhir_medication.code.coding[0].system = "http://snomed.info/sct"
        fhir_medication.status = "active"
        return fhir_medication

    def verify_fhir_instance(self, fhir_obj):
        self.assertEqual(5, len(fhir_obj.extension))
        extension_unit_price = fhir_obj.extension[0].valueMoney
        self.assertTrue(isinstance(extension_unit_price, Money))
        self.assertAlmostEqual(decimal.Decimal(self._TEST_MEDICATION_PRICE), extension_unit_price.value, places=2)
        extension_medication_type = fhir_obj.extension[1].valueCodeableConcept
        self.assertTrue(isinstance(extension_medication_type, CodeableConcept))
        self.assertEqual(self._TEST_MEDICATION_TYPE, extension_medication_type.coding[0].code)
        extension_medication_frequency = fhir_obj.extension[2].valueTiming
        self.assertTrue(isinstance(extension_medication_frequency, Timing))
        self.assertEqual(self._TEST_MEDICATION_FREQUENCY, extension_medication_frequency.repeat.period)
        self.assertEqual(int(re.sub("[^0-9]", "", self._TEST_MEDICATION_PACKAGE)), fhir_obj.amount.numerator.value)
        self.assertEqual(self._TEST_MEDICATION_PACKAGE, fhir_obj.form.text)

        extension_usage_context = fhir_obj.extension[3].extension
        for ext in extension_usage_context:
            usage_context = ext.valueUsageContext
            self.assertTrue(isinstance(usage_context, UsageContext))
            # in every valueCodeableConcept 'coding' in that extensions (related to usage_context) should be two slices
            # because in that scenario we have all patient category type assigned
            self.assertEqual(2, len(ext.valueUsageContext.valueCodeableConcept.coding))

        for identifier in fhir_obj.identifier:
            self.assertTrue(isinstance(identifier, Identifier))
            code = MedicationConverter.get_first_coding_from_codeable_concept(identifier.type).code
            if code == R4IdentifierConfig.get_fhir_claim_admin_code_type():
                self.assertEqual(self._TEST_MEDICATION_CODE, identifier.value)



from api_fhir_r4.configurations import CoverageConfiguration


class R4CoverageConfig(CoverageConfiguration):

    @classmethod
    def build_configuration(cls, cfg):
        cls.get_config().R4_fhir_identifier_type_config = cfg['R4_fhir_coverage_config']
        cls.get_config().R4_fhir_contract_config = cfg['R4_fhir_contract_config']

    @classmethod
    def get_status_policy_code(cls):
        return cls.get_config_attribute("R4_fhir_contract_config").get('fhir_contract_policy_status', "Policy")

    @classmethod
    def get_status_executable_code(cls):
        return cls.get_config_attribute("R4_fhir_contract_config").get('fhir_contract_executable_status', "Executable")

    @classmethod
    def get_status_terminated_code(cls):

        return cls.get_config_attribute("R4_fhir_contract_config").get('fhir_contract_terminated_status', "Terminated")

    @classmethod
    def get_status_disputed_code(cls):
        return cls.get_config_attribute("R4_fhir_contract_config").get('fhir_contract_disputed_status', "Disputed")

    @classmethod
    def get_status_offered_code(cls):
        return cls.get_config_attribute("R4_fhir_contract_config").get('fhir_contract_offered_status', "Offered")

    @classmethod
    def get_status_renewed_code(cls):
        return cls.get_config_attribute("R4_fhir_contract_config").get('fhir_contract_renewed_status', "Renewed")

    @classmethod
    def get_signer_eo_type_code(cls):
        return cls.get_config_attribute("R4_fhir_contract_config").get('fhir_contract_eo_signer_type', "EnrolmentOfficer")

    @classmethod
    def get_signer_head_type_code(cls):
        return cls.get_config_attribute("R4_fhir_contract_config").get('fhir_contract_head_signer_type', "HeadOfFamily")

    @classmethod
    def get_offer_insuree_role_code(cls):
        return cls.get_config_attribute("R4_fhir_contract_config").get('fhir_contract_insuree_role', "Insuree")

    @classmethod
    def get_offer_dependant_role_code(cls):
        return cls.get_config_attribute("R4_fhir_contract_config").get('fhir_contract_dependant_role', "Dependant")

    @classmethod
    def get_organization_code(cls):
        return cls.get_config_attribute("R4_fhir_contract_config").get('fhir_organization_code', "openIMIS")

    @classmethod
    def get_family_reference_code(cls):
        return cls.get_config_attribute("R4_fhir_claim_config").get('fhir_family_refereence_code', "FamilyReference")

    @classmethod
    def get_status_idle_code(cls):
        return cls.get_config_attribute("R4_fhir_claim_config").get('fhir_status_idle_code', "Idle")

    @classmethod
    def get_status_cancelled_code(cls):
        return cls.get_config_attribute("R4_fhir_claim_config").get('fhir_status_cancelled_code', "cancelled")

    @classmethod
    def get_status_draft_code(cls):
        return cls.get_config_attribute("R4_fhir_claim_config").get('fhir_status_draft_code', "draft")

    @classmethod
    def get_status_active_code(cls):
        return cls.get_config_attribute("R4_fhir_claim_config").get('fhir_status_active_code', "active")

    @classmethod
    def get_status_suspended_code(cls):
        return cls.get_config_attribute("R4_fhir_claim_config").get('fhir_status_suspended_code', "suspended")

    @classmethod
    def get_status_expired_code(cls):
        return cls.get_config_attribute("R4_fhir_claim_config").get('fhir_status_expired_code', "Expired")

    @classmethod
    def get_item_code(cls):
        return cls.get_config_attribute("R4_fhir_claim_config").get('fhir_item_code', "item")

    @classmethod
    def get_service_code(cls):
        return cls.get_config_attribute("R4_fhir_claim_config").get('fhir_service_code', "service")

    @classmethod
    def get_practitioner_role_code(cls):
        return cls.get_config_attribute("R4_fhir_claim_config").get('fhir_practitioner_role_code', "Practitioner")

    @classmethod
    def get_product_code(cls):
        return cls.get_config_attribute("R4_fhir_claim_config").get('fhir_product_code', "Product")

    @classmethod
    def get_enroll_date_code(cls):
        return cls.get_config_attribute("R4_fhir_claim_config").get('fhir_enroll_date_code', "EnrollDate")

    @classmethod
    def get_effective_date_code(cls):
        return cls.get_config_attribute("R4_fhir_claim_config").get('fhir_effective_date_code', "EffectiveDate")




import json
import os

from rest_framework import status
from rest_framework.test import APITestCase

from api_fhir_r4.configurations import GeneralConfiguration
from api_fhir_r4.tests import GenericFhirAPITestMixin
from api_fhir_r4.tests.mixin.logInMixin import LogInMixin


class CommunicationRequestAPITests(GenericFhirAPITestMixin, APITestCase, LogInMixin):
    base_url = GeneralConfiguration.get_base_url() + 'CommunicationRequest/'
    _test_json_path = "/test/test_communicationRequest.json"

    _test_json_path_credentials = "/test/test_login.json"
    _test_request_data_credentials = None

    def setUp(self):
        super(CommunicationRequestAPITests, self).setUp()
        self._TEST_USER = self.get_or_create_user_api()

    def test_get_should_return_200(self):
        # test if return 200
        response = self.client.post(
            GeneralConfiguration.get_base_url() + 'login/', data=self._test_request_data_credentials, format='json'
        )
        response_json = response.json()
        token = response_json["token"]
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        headers = {
            "Content-Type": "application/json",
            "HTTP_AUTHORIZATION": f"Bearer {token}"
        }
        response = self.client.get(self.base_url, data=None, format='json', **headers)
        self.assertEqual(response.status_code, status.HTTP_200_OK)



from api_fhir_r4.configurations import PaymentNoticeConfiguration


class R4PaymentNoticeConfig(PaymentNoticeConfiguration):
    _config = 'R4_fhir_payment_notice_config'

    @classmethod
    def build_configuration(cls, cfg):
        cls.get_config().R4_fhir_payment_notice_config = cfg['R4_fhir_payment_notice_config']

    @classmethod
    def get_fhir_payment_notice_status_active(cls):
        return cls.get_config_attribute('R4_fhir_payment_notice_config')\
            .get('fhir_payment_notice_status_active', 'active')

    @classmethod
    def get_fhir_payment_notice_status_cancelled(cls):
        return cls.get_config_attribute('R4_fhir_payment_notice_config')\
            .get('fhir_payment_notice_status_cancelled', 'cancelled')

    @classmethod
    def get_fhir_payment_notice_status_draft(cls):
        return cls.get_config_attribute('R4_fhir_payment_notice_config')\
            .get('fhir_payment_notice_status_draft', 'draft')

    @classmethod
    def get_fhir_payment_notice_status_entered_in_error(cls):
        return cls.get_config_attribute('R4_fhir_payment_notice_config')\
            .get('fhir_payment_notice_status_entered_in_error', 'entered-in-error')

    @classmethod
    def get_fhir_payment_notice_payment_status_paid(cls):
        return cls.get_config_attribute('R4_fhir_payment_notice_config') \
            .get('fhir_payment_notice_payment_status_paid', 'paid')

    @classmethod
    def get_fhir_payment_notice_payment_status_cleared(cls):
        return cls.get_config_attribute('R4_fhir_payment_notice_config') \
            .get('fhir_payment_notice_payment_status_cleared', 'cleared')



from api_fhir_r4.configurations import SubscriptionConfiguration


class R4SubscriptionConfig(SubscriptionConfiguration):
    _config = 'R4_fhir_subscription_config'

    @classmethod
    def build_configuration(cls, cfg):
        cls.get_config().R4_fhir_subscription_config = cfg['R4_fhir_subscription_config']

    @classmethod
    def get_fhir_sub_search_perms(cls):
        return cls.get_config_attribute('R4_fhir_subscription_config').get('get_fhir_sub_search_perms', ['158001'])

    @classmethod
    def get_fhir_sub_create_perms(cls):
        return cls.get_config_attribute('R4_fhir_subscription_config').get('fhir_sub_create_perms', ['158002'])

    @classmethod
    def get_fhir_sub_update_perms(cls):
        return cls.get_config_attribute('R4_fhir_subscription_config').get('get_fhir_sub_update_perms', ['158003'])

    @classmethod
    def get_fhir_sub_delete_perms(cls):
        return cls.get_config_attribute('R4_fhir_subscription_config').get('get_fhir_sub_delete_perms', ['158004'])

    @classmethod
    def get_fhir_subscription_channel_rest_hook(cls):
        return cls.get_config_attribute('R4_fhir_subscription_config').get('fhir_sub_channel_rest_hook', 'rest_hook')

    @classmethod
    def get_fhir_subscription_status_off(cls):
        return cls.get_config_attribute('R4_fhir_subscription_config').get('fhir_sub_status_off', 'off')

    @classmethod
    def get_fhir_subscription_status_active(cls):
        return cls.get_config_attribute('R4_fhir_subscription_config').get('fhir_sub_status_active', 'active')

    @classmethod
    def get_fhir_sub_criteria_key_resource(cls):
        return cls.get_config_attribute('R4_fhir_subscription_config').get('get_fhir_sub_criteria_key_resource',
                                                                           'resource')

    @classmethod
    def get_fhir_sub_criteria_key_resource_type(cls):
        return cls.get_config_attribute('R4_fhir_subscription_config').get('get_fhir_sub_criteria_key_resource_type',
                                                                           'resource_type')



import copy
import uuid

from claim.models import ClaimAdmin

from api_fhir_r4.converters import ClaimAdminPractitionerConverter
from api_fhir_r4.exceptions import FHIRException
from api_fhir_r4.serializers import BaseFHIRSerializer


class ClaimAdminPractitionerSerializer(BaseFHIRSerializer):

    fhirConverter = ClaimAdminPractitionerConverter()

    def create(self, validated_data):
        code = validated_data.get('code')
        if 'uuid' in validated_data.keys() and validated_data.get('uuid') is None:
            # In serializers using graphql services can't provide uuid. If uuid is provided then
            # resource is updated and not created. This check ensure UUID was provided.
            validated_data['uuid'] = uuid.uuid4()

        if ClaimAdmin.objects.filter(code=code).count() > 0:
            raise FHIRException('Exists practitioner with following code `{}`'.format(code))
        copied_data = copy.deepcopy(validated_data)
        del copied_data['_state']
        return ClaimAdmin.objects.create(**copied_data)

    def update(self, instance, validated_data):
        instance.code = validated_data.get('code', instance.code)
        instance.last_name = validated_data.get('last_name', instance.last_name)
        instance.other_names = validated_data.get('other_names', instance.other_names)
        instance.dob = validated_data.get('dob', instance.dob)
        instance.phone = validated_data.get('phone', instance.phone)
        instance.email_id = validated_data.get('email_id', instance.email_id)
        instance.audit_user_id = self.get_audit_user_id()
        instance.save()
        return instance



import json
import os
from unittest import skip

from django.utils.translation import gettext as _
from fhir.resources.R4B.patient import Patient
from rest_framework import status
from rest_framework.test import APITestCase

from api_fhir_r4.configurations import GeneralConfiguration
from api_fhir_r4.tests import GenericFhirAPITestMixin, PatientTestMixin, FhirApiReadTestMixin
from api_fhir_r4.tests.utils import get_or_create_user_api,load_and_replace_json ,get_connection_payload
from insuree.models import Gender
from insuree.test_helpers import create_test_insuree
from location.test_helpers import create_test_village

class PatientAPITests(GenericFhirAPITestMixin, FhirApiReadTestMixin, APITestCase):
    base_url = GeneralConfiguration.get_base_url() + 'Patient/'
    _json_repr = "/test/test_patient.json"
    _TEST_LAST_NAME = "TEST_LAST_NAME"
    _TEST_VILLAGE_NAME = "Rachla"
    _TEST_GENDER_CODE = 'M'
    _TEST_EXPECTED_NAME = "UPDATED_NAME"
    _TEST_INSUREE_UUID = "7240daef-5f8f-4b0f-9042-b221e66f184a"
    _TEST_GROUP_UUID = "8e33033a-9f60-43ad-be3e-3bfeb992aae5"
    _TEST_VILLAGE_UUID = "69a55f2d-ee34-4193-be0e-2b6a361797bd"

    _test_json_path_credentials = "/test/test_login.json"
    _test_request_data_credentials = None
    test_village = None
    test_user = None
    sub_str = {}
    def setUp(self):
        super(PatientAPITests, self).setUp()
        self.test_user = get_or_create_user_api()
        self.test_village = create_test_village()
        self.sub_str[self._TEST_VILLAGE_UUID] = self.test_village.uuid

        
        self._test_request_data = load_and_replace_json(self._json_repr,self.sub_str)

    def verify_updated_obj(self, updated_obj):
        self.assertTrue(isinstance(updated_obj, Patient))
        self.assertEqual(self._TEST_EXPECTED_NAME, updated_obj.name[0].given[0])

    def update_resource(self, data):
        data['name'][0]['given'][0] = self._TEST_EXPECTED_NAME



    def update_payload_missing_chfid_identifier(self, data):
        for i in range(len(data["identifier"])):
            if data["identifier"][i]["type"]["coding"][0]["code"] == "Code":
                del data["identifier"][i]
                return data

    def update_payload_no_extensions(self, data):
        data["extension"] = []
        return data

    def update_payload_missing_fhir_address_details(self, data, field, kind_of_address):
        for address in data["address"]:
            if address["use"] == kind_of_address:
                address.pop(field)
        return data

    def update_payload_missing_fhir_address_extension(self, data, kind_of_extension):
        for address in data["address"]:
            if address["use"] == "home":
                for i in range(len(address["extension"])):
                    if kind_of_extension in address["extension"][i]['url']:
                        del address["extension"][i]
                        return data

    def update_payload_missing_fhir_address_extensions_all(self, data):
        for address in data["address"]:
            if address["use"] == "home":
                for i in range(len(address["extension"])):
                    address.pop("extension")
                    return data

    def update_payload_fhir_no_address(self, data):
        data["address"] = []
        return data

    def update_payload_fhir_address_no_photo(self, data):
        data.pop("photo")
        return data

    def update_payload_fhir_address_missing_photo_data(self, data):
        for photo in data["photo"]:
            photo.pop("title")
        return data

    def update_payload_fhir_address_no_name(self, data):
        data.pop("name")
        return data

    def update_payload_fhir_address_missing_name_given_field(self, data):
        for name in data["name"]:
            name.pop('given')
        return data

    def test_post_should_create_correctly(self):
        response = self.client.post(
            GeneralConfiguration.get_base_url() + 'login/', data=get_connection_payload(), format='json'
        )
        response_json = response.json()
        token = response_json["token"]
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        headers = {
            "Content-Type": "application/json",
            'HTTP_AUTHORIZATION': f"Bearer {token}"
        }
        response = self.client.post(self.base_url, data=self._test_request_data, format='json', **headers)
        #FIXME invalid location ref self.assertEqual(response.status_code, status.HTTP_201_CREATED)
        #self.assertIsNotNone(response.content)

    def test_post_should_raise_error_no_chfid_identifier(self):
        response = self.client.post(
            GeneralConfiguration.get_base_url() + 'login/', data=get_connection_payload(), format='json'
        )
        response_json = response.json()
        token = response_json["token"]
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        headers = {
            "Content-Type": "application/json",
            'HTTP_AUTHORIZATION': f"Bearer {token}"
        }
        modified_payload = self.update_payload_missing_chfid_identifier(data=self._test_request_data)
        response = self.client.post(self.base_url, data=modified_payload, format='json', **headers)
        response_json = response.json()
        if 'issue' in response_json and len(response_json["issue"])>0\
            and 'details'in response_json["issue"][0]\
            and 'severity' in response_json["issue"][0]['details']:
            severity = response_json["issue"][0]['details']['severity']
        elif 'detail' in response_json and response_json['detail'] == 'Patient code not provided.':
            severity = 'error'
        else:
            severity =  f"no error in {response.content}"
        self.assertTrue(response.status_code, 500)
        self.assertEqual(severity, 'error')

    def test_post_should_raise_error_no_extensions(self):
        self.login()
        modified_payload = self.update_payload_no_extensions(data=self._test_request_data)
        response = self.client.post(self.base_url, data=modified_payload, format='json')
        response_json = response.json()
        self.assertEqual(
            self.get_response_details(response_json),
            _("At least one extension with is_head is required")
        )

    def test_post_should_raise_missing_fhir_home_address_details(self):
        self.login()
        # missing city
        self._assert_filed_mandatory('city')
        self._assert_filed_mandatory('district')
        self._assert_filed_mandatory('state')

    def test_post_should_raise_missing_fhir_address_home_family_extensions(self):
        self.login()
        # missing municipality extension
        modified_payload = self.update_payload_missing_fhir_address_extension(
            data=self._test_request_data, kind_of_extension='address-municipality')
        response = self.client.post(self.base_url, data=modified_payload, format='json')
        response_json_municipality = response.json()
        # missing all extensions
        modified_payload = self.update_payload_missing_fhir_address_extensions_all(data=self._test_request_data)
        response = self.client.post(self.base_url, data=modified_payload, format='json')
        response_json_no_extensions = response.json()
        self.assertIn(
            "FHIR Patient address without address-municipality reference.",
            self.get_response_details(response_json_municipality),
        )

        self.assertEqual(
            self.get_response_details(response_json_no_extensions),
            _("Missing extensions for Address")
        )

    @skip("This test needs to be checked. Isnuree without family can have no address")
    def test_post_should_raise_error_no_address(self):
        self.login()
        modified_payload = self.update_payload_fhir_no_address(data=self._test_request_data)
        response = self.client.post(self.base_url, data=modified_payload, format='json')
        response_json = response.json()
        self.assertEqual(
            self.get_response_details(response_json),
            _('Address must be supported')
        )

    @skip("This test needs to be checked. At the moment photo is not obligatory")
    def test_post_should_raise_error_no_photo(self):
        self.login()
        modified_payload = self.update_payload_fhir_address_no_photo(data=self._test_request_data)
        response = self.client.post(self.base_url, data=modified_payload, format='json')
        response_json = response.json()
        self.assertEqual(
            self.get_response_details(response_json),
            _('FHIR Patient without photo data.')
        )

    def test_post_should_raise_error_missing_photo_data(self):
        self.login()
        modified_payload = self.update_payload_fhir_address_missing_photo_data(data=self._test_request_data)
        response = self.client.post(self.base_url, data=modified_payload, format='json')
        response_json = response.json()
        self.assertEqual(
            self.get_response_details(response_json),
            _('FHIR Patient misses one of required fields:  contentType, title, creation')
        )

    def test_post_should_raise_error_missing_name_attribute(self):
        self.login()
        modified_payload = self.update_payload_fhir_address_missing_name_given_field(data=self._test_request_data)
        response = self.client.post(self.base_url, data=modified_payload, format='json')
        response_json_no_given_name = response.json()
        self.assertEqual(
            self.get_response_details(response_json_no_given_name),
            _('Missing obligatory fields for fhir patient name: family or given')
        )
        modified_payload = self.update_payload_fhir_address_no_name(data=self._test_request_data)
        response = self.client.post(self.base_url, data=modified_payload, format='json')
        response_json_no_name = response.json()
        self.assertEqual(
            self.get_response_details(response_json_no_name),
            _('Missing fhir patient attribute: name')
        )

    def _assert_filed_mandatory(self, field):
        modified_payload = self.update_payload_missing_fhir_address_details(
            data=self._test_request_data, field=field, kind_of_address="home")
        response = self.client.post(self.base_url, data=modified_payload, format='json')
        json_response = response.json()

        # Missing mandatory field should result in operation failure.
        self.assertEqual(response.status_code, 500)
        # Information regarding failure reason should be provided
        self.assertIsNotNone(self.get_response_details(json_response))
        # Information regarding field should be part of failure reason
        self.assertIn(field, self.get_response_details(json_response))



from api_fhir_r4.converters import EnrolmentOfficerPractitionerConverter

from fhir.resources.R4B.practitioner import Practitioner
from api_fhir_r4.tests import EnrolmentOfficerPractitionerTestMixin
from api_fhir_r4.tests.mixin import ConvertToImisTestMixin, ConvertToFhirTestMixin, ConvertJsonToFhirTestMixin


class EnrolmentOfficerPractitionerConverterTestCase(EnrolmentOfficerPractitionerTestMixin,
                                                    ConvertToImisTestMixin,
                                                    ConvertToFhirTestMixin,
                                                    ConvertJsonToFhirTestMixin):
    converter = EnrolmentOfficerPractitionerConverter
    fhir_resource = Practitioner
    json_repr = 'test/test_enrolmentOfficerPractitioner.json'



# Generated by Django 3.2.16 on 2023-05-04 11:56
import logging

from django.db import migrations
from core.models import RoleRight
from core.utils import insert_role_right_for_system

logger = logging.getLogger(__name__)

IMIS_ADMIN_ROLE_IS_SYSTEM = 64
SUB_SEARCH_ROLE_RIGHT_ID = 158001
SUB_CREATE_ROLE_RIGHT_ID = 158002
SUB_UPDATE_ROLE_RIGHT_ID = 158003
SUB_DELETE_ROLE_RIGHT_ID = 158004


def add_rights(apps, schema_editor):
    """
    Add subscription CRUD permission to the IMIS Administrator.
    """
    insert_role_right_for_system(IMIS_ADMIN_ROLE_IS_SYSTEM, SUB_SEARCH_ROLE_RIGHT_ID)
    insert_role_right_for_system(IMIS_ADMIN_ROLE_IS_SYSTEM, SUB_CREATE_ROLE_RIGHT_ID)
    insert_role_right_for_system(IMIS_ADMIN_ROLE_IS_SYSTEM, SUB_UPDATE_ROLE_RIGHT_ID)
    insert_role_right_for_system(IMIS_ADMIN_ROLE_IS_SYSTEM, SUB_DELETE_ROLE_RIGHT_ID)


def remove_rights(apps, schema_editor):
    """
    Remove subscription CRUD permissions to the IMIS Administrator.
    """
    RoleRight.objects.filter(
        role__is_system=IMIS_ADMIN_ROLE_IS_SYSTEM,
        right_id__in=[SUB_CREATE_ROLE_RIGHT_ID, SUB_DELETE_ROLE_RIGHT_ID,
                      SUB_SEARCH_ROLE_RIGHT_ID, SUB_UPDATE_ROLE_RIGHT_ID],
        validity_to__isnull=True
    ).delete()


class Migration(migrations.Migration):

    dependencies = [
        ('api_fhir_r4', '0005_auto_20221012_0818'),
    ]

    operations = [
        migrations.RunPython(add_rights, remove_rights),
    ]



from api_fhir_r4.configurations import GeneralConfiguration
from api_fhir_r4.converters import CommunicationConverter
from api_fhir_r4.configurations import R4CommunicationRequestConfig as Config
from api_fhir_r4.tests import GenericTestMixin, LocationTestMixin
from api_fhir_r4.utils import TimeUtils
from fhir.resources.R4B.communication import Communication, CommunicationPayload
from fhir.resources.R4B.extension import Extension
from claim.models import Claim, ClaimItem, ClaimService, Feedback
from claim.test_helpers import create_test_claim_admin
from core import datetime
from location.models import HealthFacility
from location.test_helpers import create_test_village, create_test_health_facility
from insuree.test_helpers import create_test_insuree
from medical.test_helpers import create_test_item, create_test_service
from medical.models import Diagnosis


class CommunicationTestMixin(GenericTestMixin):
    # feedback expected data
    _TEST_FEE_UUID = "612a1e12-ce44-4632-90a8-129ec714ec59"
    _TEST_CARE_RENDERED = False
    _TEST_PAYMENT_ASKED = True
    _TEST_DRUG_PRESCRIBED = True
    _TEST_DRUG_RECEIVED = False
    _TEST_ASESSMENT = '3'

    # claim data
    _TEST_CLAIM_CODE = 'codeTest'
    _TEST_CLAIM_UUID = "7ac646cb-d3cd-4660-baeb-ee34ecf0354e"
    _TEST_STATUS = Claim.STATUS_ENTERED
    _TEST_STATUS_DISPLAY = "entered"
    _TEST_OUTCOME = "queued"
    _TEST_ADJUSTMENT = "adjustment"
    _TEST_DATE_PROCESSED = "2010-11-16T00:00:00"
    _TEST_APPROVED = 1000.00
    _TEST_REJECTION_REASON = 0
    _TEST_VISIT_TYPE = "O"

    # claim item data
    _TEST_ITEM_CODE = "iCode"
    _TEST_ITEM_UUID = "e2bc1546-390b-4d41-8571-632ecf7a0936"
    _TEST_ITEM_STATUS = Claim.STATUS_ENTERED
    _TEST_ITEM_QUANTITY = 20
    _TEST_ITEM_PRICE = 10.0
    _TEST_ITEM_REJECTED_REASON = 0

    # claim service data
    _TEST_SERVICE_CODE = "sCode"
    _TEST_SERVICE_UUID = "a17602f4-e9ff-4f42-a6a4-ccefcb23b4d6"
    _TEST_SERVICE_STATUS = Claim.STATUS_ENTERED
    _TEST_SERVICE_QUANTITY = 1
    _TEST_SERVICE_PRICE = 800
    _TEST_SERVICE_REJECTED_REASON = 0

    _TEST_ID = 9999
    _PRICE_ASKED = 1000
    _PRICE_APPROVED = 1000
    _ADMIN_AUDIT_USER_ID = 1

    _TEST_ITEM_AVAILABILITY = True

    _TEST_ITEM_TYPE = 'D'
    _TEST_SERVICE_TYPE = 'D'

    # insuree and claim admin data
    _TEST_PATIENT_UUID = "76aca309-f8cf-4890-8f2e-b416d78de00b"
    _TEST_CLAIM_ADMIN_UUID = "044c33d1-dbf3-4d6a-9924-3797b461e535"

    # hf test data
    _TEST_HF_ID = 10000
    _TEST_HF_UUID = "6d0eea8c-62eb-11ea-94d6-c36229a16c2f"
    _TEST_HF_CODE = "12345678"
    _TEST_HF_NAME = "TEST_NAME"
    _TEST_HF_LEVEL = "H"
    _TEST_HF_LEGAL_FORM = "G"
    _TEST_ADDRESS = "TEST_ADDRESS"
    _TEST_PHONE = "133-996-476"
    _TEST_FAX = "1-408-999 8888"
    _TEST_EMAIL = "TEST@TEST.com"
    _TEST_INSUREE = None

    def setUp(self):
        super(CommunicationTestMixin, self).setUp()
        self._TEST_INSUREE = create_test_insuree()
        self._TEST_CLAIM = self.create_test_claim()
        self._TEST_ITEM = self.create_test_claim_item()
        self._TEST_SERVICE = self.create_test_claim_service()

    def create_test_claim_item(self):
        item = ClaimItem()
        item.item = create_test_item(
            self._TEST_ITEM_TYPE,
            custom_props={"code": self._TEST_ITEM_CODE}
        )
        item.claim = self._TEST_CLAIM
        item.status = self._TEST_ITEM_STATUS
        item.qty_approved = self._TEST_ITEM_QUANTITY
        item.qty_provided = self._TEST_ITEM_QUANTITY
        item.rejection_reason = self._TEST_ITEM_REJECTED_REASON
        item.availability = self._TEST_ITEM_AVAILABILITY
        item.price_asked = self._TEST_ITEM_PRICE
        item.price_approved = self._TEST_ITEM_PRICE
        item.audit_user_id = self._ADMIN_AUDIT_USER_ID
        item.save()
        return item

    def create_test_claim_service(self):
        service = ClaimService()
        service.service = create_test_service(
            self._TEST_SERVICE_TYPE,
            custom_props={"code": self._TEST_SERVICE_CODE}
        )
        service.claim = self._TEST_CLAIM
        service.status = self._TEST_SERVICE_STATUS
        service.qty_approved = self._TEST_SERVICE_QUANTITY
        service.qty_provided = self._TEST_SERVICE_QUANTITY
        service.rejection_reason = self._TEST_SERVICE_REJECTED_REASON
        service.availability = self._TEST_ITEM_AVAILABILITY
        service.price_asked = self._TEST_SERVICE_PRICE
        service.price_approved = self._TEST_SERVICE_PRICE
        service.audit_user_id = self._ADMIN_AUDIT_USER_ID
        service.save()
        return service

    def create_test_hf(self):
        hf = create_test_health_facility(
            self._TEST_HF_CODE,
            self._TEST_INSUREE.family.location.parent.parent.id,
            custom_props = {
                'name': self._TEST_HF_NAME,
                'level':self._TEST_HF_LEVEL,
                'legal_form_id':self._TEST_HF_LEGAL_FORM,
                'address':self._TEST_ADDRESS,
                'phone':self._TEST_PHONE,
                'fax':self._TEST_FAX,
                'email':self._TEST_EMAIL,
            }
        )    

        return hf

    def create_test_claim(self):
        imis_claim = Claim()
        imis_claim.id = self._TEST_ID
        imis_claim.uuid = self._TEST_CLAIM_UUID
        imis_claim.code = self._TEST_CLAIM_CODE
        imis_claim.status = self._TEST_STATUS
        imis_claim.adjustment = self._TEST_ADJUSTMENT
        imis_claim.date_processed = TimeUtils.str_to_date(self._TEST_DATE_PROCESSED)
        imis_claim.approved = self._TEST_APPROVED
        imis_claim.rejection_reason = self._TEST_REJECTION_REASON
        imis_claim.insuree = self._TEST_INSUREE
        imis_claim.insuree.uuid = self._TEST_PATIENT_UUID
        imis_claim.insuree.save()
        imis_claim.health_facility = self.create_test_hf()
        imis_claim.icd = Diagnosis(code='ICD00I', name='Test ICD')
        imis_claim.icd.audit_user_id = self._ADMIN_AUDIT_USER_ID
        imis_claim.icd.save()
        imis_claim.audit_user_id = self._ADMIN_AUDIT_USER_ID
        imis_claim.icd.date_from = datetime.date(2018, 12, 12)
        imis_claim.date_from = datetime.date(2018, 12, 12)
        imis_claim.date_claimed = datetime.date(2018, 12, 14)
        imis_claim.visit_type = self._TEST_VISIT_TYPE
        claim_admin = create_test_claim_admin()
        claim_admin.uuid = self._TEST_CLAIM_ADMIN_UUID
        claim_admin.save()
        imis_claim.admin = claim_admin
        imis_claim.feedback_status = Claim.FEEDBACK_SELECTED
        imis_claim.save()
        return imis_claim

    def create_test_imis_instance(self):
        imis_feedback = Feedback()
        imis_feedback.claim = self._TEST_CLAIM
        imis_feedback.care_rendered = self._TEST_CARE_RENDERED
        imis_feedback.payment_asked = self._TEST_PAYMENT_ASKED
        imis_feedback.drug_prescribed = self._TEST_DRUG_PRESCRIBED
        imis_feedback.drug_received = self._TEST_DRUG_RECEIVED
        imis_feedback.asessment = self._TEST_ASESSMENT
        return imis_feedback

    def verify_imis_instance(self, imis_obj):
        self.assertEqual(imis_obj.claim.uuid.lower(), self._TEST_CLAIM_UUID.lower())
        self.assertEqual(imis_obj.care_rendered, self._TEST_CARE_RENDERED)
        self.assertEqual(imis_obj.payment_asked, self._TEST_PAYMENT_ASKED)
        self.assertEqual(imis_obj.drug_prescribed, self._TEST_DRUG_PRESCRIBED)
        self.assertEqual(imis_obj.asessment, self._TEST_ASESSMENT)

    def create_test_fhir_instance(self):
        fhir_communication = {}
        fhir_communication['status'] = "completed"

        fhir_communication = Communication(**fhir_communication)

        fhir_payload = []
        # care rendered
        payload = {}
        payload['contentString'] = "no"
        payload = CommunicationPayload(**payload)
        payload.extension = []

        extension = Extension.construct()
        url = f'{GeneralConfiguration.get_system_base_url()}StructureDefinition/communication-payload-type'
        system = f'{GeneralConfiguration.get_system_base_url()}CodeSystem/feedback-payload'
        extension.url = url
        extension.valueCodeableConcept = CommunicationConverter.build_codeable_concept(
            system=system,
            code=Config.get_fhir_care_rendered_code()
        )
        payload.extension.append(extension)

        fhir_payload.append(payload)

        # payment asked
        payload = {}
        payload['contentString'] = "yes"
        payload = CommunicationPayload(**payload)
        payload.extension = []

        extension = Extension.construct()
        extension.url = url
        extension.valueCodeableConcept = CommunicationConverter.build_codeable_concept(
            system=system,
            code=Config.get_fhir_payment_asked_code()
        )
        payload.extension.append(extension)

        fhir_payload.append(payload)

        # drug prescribed
        payload = {}
        payload['contentString'] = "yes"
        payload = CommunicationPayload(**payload)
        payload.extension = []

        extension = Extension.construct()
        extension.url = url
        extension.valueCodeableConcept = CommunicationConverter.build_codeable_concept(
            system=system,
            code=Config.get_fhir_drug_prescribed_code()
        )
        payload.extension.append(extension)

        fhir_payload.append(payload)

        # drug received
        payload = {}
        payload['contentString'] = "no"
        payload = CommunicationPayload(**payload)
        payload.extension = []
        extension = Extension.construct()
        extension.url = url
        extension.valueCodeableConcept = CommunicationConverter.build_codeable_concept(
            system=system,
            code=Config.get_fhir_drug_received_code()
        )
        payload.extension.append(extension)
        fhir_payload.append(payload)

        # assesment
        payload = {}
        payload['contentString'] = self._TEST_ASESSMENT
        payload = CommunicationPayload(**payload)
        payload.extension = []

        extension = Extension.construct()
        extension.url = url
        extension.valueCodeableConcept = CommunicationConverter.build_codeable_concept(
            system=system,
            code=Config.get_fhir_asessment_code()
        )
        payload.extension.append(extension)

        fhir_payload.append(payload)

        fhir_communication.payload = fhir_payload

        fhir_communication.about = [CommunicationConverter.build_fhir_resource_reference(
            self._TEST_CLAIM,
            type="Claim",
        )]

        fhir_communication.subject = CommunicationConverter.build_fhir_resource_reference(
            self._TEST_CLAIM.insuree,
            type="Patient",
        )

        return fhir_communication

    def verify_fhir_instance(self, fhir_obj):
        self.assertEqual("completed", fhir_obj.status)
        self.assertIn(self._TEST_PATIENT_UUID, fhir_obj.subject.reference)
        self.assertIn(self._TEST_CLAIM_UUID, fhir_obj.about[0].reference)
        for payload in fhir_obj.payload:
            code = payload.extension[0].valueCodeableConcept.coding[0].code
            content_string = payload.contentString
            if code != Config.get_fhir_asessment_code():
                bool_value = self._convert_bool_value(content_string)
            if code == Config.get_fhir_care_rendered_code():
                self.assertEqual(self._TEST_CARE_RENDERED, bool_value)
            elif code == Config.get_fhir_payment_asked_code():
                self.assertEqual(self._TEST_PAYMENT_ASKED, bool_value)
            elif code == Config.get_fhir_drug_prescribed_code():
                self.assertEqual(self._TEST_DRUG_PRESCRIBED, bool_value)
            elif code == Config.get_fhir_drug_received_code():
                self.assertEqual(self._TEST_DRUG_RECEIVED, bool_value)
            elif code == Config.get_fhir_asessment_code():
                self.assertEqual(self._TEST_ASESSMENT, content_string)

    def _convert_bool_value(self, fhir_content_string):
        if fhir_content_string == "yes":
            return True
        if fhir_content_string == "no":
            return False
        return None



from api_fhir_r4.converters import CommunicationRequestConverter
from api_fhir_r4.serializers import BaseFHIRSerializer


class CommunicationRequestSerializer(BaseFHIRSerializer):

    fhirConverter = CommunicationRequestConverter



from rest_framework.exceptions import APIException
from django.utils.translation import gettext


class FHIRException(APIException):
    def __init__(self, message):
        super(FHIRException, self).__init__(message)


class FHIRRequestProcessException(FHIRException):
    def __init__(self, errors):
        base_massage = gettext("The request cannot be processed due to the following issues:\n")
        message = base_massage + ",\n".join(errors)
        super(FHIRRequestProcessException, self).__init__(message)


class InvalidAttributeError(FHIRException):
    def __init__(self, attr, property_type):
        msg = gettext("The attribute named '{}' is not a valid property for '{}'.").format(attr, property_type)
        super(InvalidAttributeError, self).__init__(msg)


class PropertyError(FHIRException):
    def __init__(self, message):
        super(PropertyError, self).__init__(message)


class PropertyMaxSizeError(PropertyError):
    def __init__(self, definition):
        message = gettext("The max size was exceeded for property {} [{}..{}]").format(definition.name,
                                                                              definition.count_min,
                                                                              definition.count_max)
        super(PropertyMaxSizeError, self).__init__(message)


class PropertyTypeError(Exception):
    def __init__(self, local_type, description):
        msg = gettext("Expected '{}' but got '{}' for '{}' property").format(description.type, local_type, description.name)
        super(PropertyTypeError, self).__init__(msg)


class UnsupportedFormatError(Exception):
    def __init__(self, data_format):
        message = gettext("The format '{}' is not supported").format(data_format)
        super(UnsupportedFormatError, self).__init__(message)


from api_fhir_r4.exceptions.fhir_api_exception_handler import fhir_api_exception_handler



from api_fhir_r4.configurations import GeneralConfiguration, \
    R4CommunicationRequestConfig as Config
from api_fhir_r4.converters import BaseFHIRConverter, ReferenceConverterMixin
from api_fhir_r4.mapping.feedbackMapping import FeedbackStatus
from api_fhir_r4.utils import DbManagerUtils
from claim.models import Claim
from fhir.resources.R4B.communicationrequest import CommunicationRequest, CommunicationRequestPayload
from fhir.resources.R4B.extension import Extension


class CommunicationRequestConverter(BaseFHIRConverter, ReferenceConverterMixin):

    @classmethod
    def to_fhir_obj(cls, imis_claim, reference_type=ReferenceConverterMixin.UUID_REFERENCE_TYPE):
        fhir_communication_request = CommunicationRequest.construct()
        fhir_communication_request.status = "active"
        cls.build_fhir_status(fhir_communication_request, imis_claim)
        cls.build_fhir_status_reason(fhir_communication_request, imis_claim)
        cls.build_fhir_subject(fhir_communication_request, imis_claim, reference_type)
        cls.build_fhir_about(fhir_communication_request, imis_claim, reference_type)
        cls.build_fhir_recipient(fhir_communication_request, imis_claim, reference_type)
        cls.build_fhir_payloads(fhir_communication_request)
        return fhir_communication_request

    @classmethod
    def get_reference_obj_id(cls, imis_claim):
        return imis_claim.uuid

    @classmethod
    def get_reference_obj_code(cls, imis_claim):
        return imis_claim.code

    @classmethod
    def get_fhir_resource_type(cls):
        return CommunicationRequest

    @classmethod
    def get_imis_obj_by_fhir_reference(cls, reference, errors=None):
        return DbManagerUtils.get_object_or_none(
            Claim,
            **cls.get_database_query_id_parameteres_from_reference(reference))


    @classmethod
    def get_reference_obj_uuid(cls, imis_claim):
        return imis_claim.uuid

    @classmethod
    def build_fhir_payloads(cls, fhir_communication_request):
        fhir_communication_request.payload = []
        cls.build_fhir_payload(
            fhir_communication_request.payload,
            Config.get_fhir_care_rendered_code(),
            "Care Rendered? (yes|no)"
        )
        cls.build_fhir_payload(
            fhir_communication_request.payload,
            Config.get_fhir_payment_asked_code(),
            "Payment Asked? (yes|no)"
        )
        cls.build_fhir_payload(
            fhir_communication_request.payload,
            Config.get_fhir_drug_prescribed_code(),
            "Drug Prescribed? (yes|no)"
        )
        cls.build_fhir_payload(
            fhir_communication_request.payload,
            Config.get_fhir_drug_received_code(),
            "Drug Received? (yes|no)"
        )
        cls.build_fhir_payload(
            fhir_communication_request.payload,
            Config.get_fhir_asessment_code(),
            "Asessment? (0|1|2|3|4|5)"
        )

    @classmethod
    def build_fhir_payload(cls, fhir_payload, code, content_string):
        payload = {}
        payload['contentString'] = content_string
        payload = CommunicationRequestPayload(**payload)
        payload.extension = []

        extension = Extension.construct()
        url = f'{GeneralConfiguration.get_system_base_url()}StructureDefinition/communication-payload-type'
        system = f'{GeneralConfiguration.get_system_base_url()}CodeSystem/feedback-payload'
        extension.url = url
        extension.valueCodeableConcept = cls.build_codeable_concept(
            system=system,
            code=code
        )
        payload.extension.append(extension)

        fhir_payload.append(payload)

    @classmethod
    def build_fhir_status(cls, fhir_communication_request, imis_claim):
        feedback_code = FeedbackStatus.map_status(imis_claim.feedback_status)
        fhir_communication_request.status = feedback_code

    @classmethod
    def build_fhir_status_reason(cls, fhir_communication_request, imis_claim):
        display = FeedbackStatus.map_code_display(imis_claim.feedback_status)
        system = f'{GeneralConfiguration.get_system_base_url()}CodeSystem/feedback-status'
        fhir_communication_request.statusReason = cls.build_codeable_concept(
            system=system,
            code=imis_claim.feedback_status,
            display=display
        )

    @classmethod
    def build_fhir_subject(cls, fhir_communication_request, imis_claim, reference_type):
        fhir_communication_request.subject = cls.build_fhir_resource_reference(
            imis_claim.insuree,
            reference_type=reference_type,
            type="Patient",
            display=imis_claim.insuree.chf_id
        )

    @classmethod
    def build_fhir_about(cls, fhir_communication_request, imis_claim, reference_type):
        fhir_communication_request.about = []
        reference = cls.build_fhir_resource_reference(
            imis_claim,
            reference_type=reference_type,
            type="Claim",
            display=imis_claim.code
        )
        fhir_communication_request.about.append(reference)

    @classmethod
    def build_fhir_recipient(cls, fhir_communication_request, imis_claim, reference_type):
        fhir_communication_request.recipient = []
        reference = cls.build_fhir_resource_reference(
            imis_claim.admin,
            reference_type=reference_type,
            type="Practitioner",
            display=imis_claim.admin.code
        )
        fhir_communication_request.recipient.append(reference)



from api_fhir_r4.configurations import R4SubscriptionConfig
from api_fhir_r4.models import Subscription


class SubscriptionStatusMapping:
    _IMIS_OFF = Subscription.SubscriptionStatus.INACTIVE.value
    _FHIR_OFF = R4SubscriptionConfig.get_fhir_subscription_status_off()
    _IMIS_ACTIVE = Subscription.SubscriptionStatus.ACTIVE.value
    _FHIR_ACTIVE = R4SubscriptionConfig.get_fhir_subscription_status_active()

    to_fhir_status = {
        _IMIS_OFF: _FHIR_OFF,
        _IMIS_ACTIVE: _FHIR_ACTIVE
    }

    to_imis_status = {
        _FHIR_OFF: _IMIS_OFF,
        _FHIR_ACTIVE: _IMIS_ACTIVE
    }


class SubscriptionChannelMapping:
    _IMIS_REST_HOOK = Subscription.SubscriptionChannel.REST_HOOK.value
    _FHIR_REST_HOOK = R4SubscriptionConfig.get_fhir_subscription_channel_rest_hook()

    to_fhir_channel = {
        _IMIS_REST_HOOK: _FHIR_REST_HOOK
    }

    to_imis_channel = {
        _FHIR_REST_HOOK: _IMIS_REST_HOOK
    }



from api_fhir_r4.models import Subscription
from api_fhir_r4.validation import SubscriptionValidation
from core.services import BaseService


class SubscriptionService(BaseService):
    OBJECT_TYPE = Subscription

    def __init__(self, user, validation_class=SubscriptionValidation):
        super().__init__(user, validation_class)



from api_fhir_r4.converters import PolicyHolderOrganisationConverter
from api_fhir_r4.tests.mixin import ConvertToFhirTestMixin
from api_fhir_r4.tests.mixin.policyHolderOrganisationTestMixin import PolicyHolderOrganisationTestMixin


class PolicyHolderOrganisationConverterTestCase(PolicyHolderOrganisationTestMixin,
                                                ConvertToFhirTestMixin):
    converter = PolicyHolderOrganisationConverter



import json
from api_fhir_r4.configurations import CoverageEligibilityConfiguration


class R4CoverageEligibilityConfiguration(CoverageEligibilityConfiguration):

    @classmethod
    def build_configuration(cls, cfg):
        cls.get_config().R4_fhir_coverage_eligibility_config = cfg['R4_fhir_coverage_eligibility_config']

    @classmethod
    def get_serializer(cls):
        return cls.get_config_attribute("R4_fhir_coverage_eligibility_config").get('fhir_serializer')

    @classmethod
    def get_fhir_financial_code(cls):
        pass

    @classmethod
    def get_fhir_item_code(cls):
        return cls.get_config_attribute("R4_fhir_coverage_eligibility_config").get('fhir_item_code', 'item')

    @classmethod
    def get_fhir_service_code(cls):
        return cls.get_config_attribute("R4_fhir_coverage_eligibility_config").get('fhir_service_code', 'service')

    @classmethod
    def get_fhir_total_admissions_code(cls):
        return cls.get_config_attribute("R4_fhir_coverage_eligibility_config").get('fhir_total_admissions_code', 'total_admissions')

    @classmethod
    def get_fhir_total_visits_code(cls):
        return cls.get_config_attribute("R4_fhir_coverage_eligibility_config").get('fhir_total_visits_code', 'total_visits')

    @classmethod
    def get_fhir_total_consultations_code(cls):
        return cls.get_config_attribute("R4_fhir_coverage_eligibility_config").get('fhir_total_consultations_code', 'total_consultations')

    @classmethod
    def get_fhir_total_surgeries_code(cls):
        return cls.get_config_attribute("R4_fhir_coverage_eligibility_config").get('fhir_total_surgeries_code', 'total_surgeries')

    @classmethod
    def get_fhir_total_deliveries_code(cls):
        return cls.get_config_attribute("R4_fhir_coverage_eligibility_config").get('fhir_total_deliveries_code', 'total_deliveries')

    @classmethod
    def get_fhir_total_antenatal_code(cls):
        return cls.get_config_attribute("R4_fhir_coverage_eligibility_config").get('fhir_total_antenatal_code', 'total_antenatal')

    @classmethod
    def get_fhir_consultation_amount_code(cls):
        return cls.get_config_attribute("R4_fhir_coverage_eligibility_config").get('fhir_consultation_amount_code',
                                                                 'consultation_amount')

    @classmethod
    def get_fhir_surgery_amount_code(cls):
        return cls.get_config_attribute("R4_fhir_coverage_eligibility_config").get('fhir_surgery_amount_code', 'surgery_amount')

    @classmethod
    def get_fhir_delivery_amount_code(cls):
        return cls.get_config_attribute("R4_fhir_coverage_eligibility_config").get('fhir_delivery_amount_code', 'delivery_amount')

    @classmethod
    def get_fhir_hospitalization_amount_code(cls):
        return cls.get_config_attribute("R4_fhir_coverage_eligibility_config").get('fhir_hospitalization_amount_code',
                                                                 'hospitalization_amount')

    @classmethod
    def get_fhir_antenatal_amount_code(cls):
        return cls.get_config_attribute("R4_fhir_coverage_eligibility_config").get('fhir_antenatal_amount_code', 'antenatal_amount')

    @classmethod
    def get_fhir_service_left_code(cls):
        return cls.get_config_attribute("R4_fhir_coverage_eligibility_config").get('fhir_service_left_code', 'service_left')

    @classmethod
    def get_fhir_item_left_code(cls):
        return cls.get_config_attribute("R4_fhir_coverage_eligibility_config").get('fhir_item_left_code', 'item_left')

    @classmethod
    def get_fhir_is_item_ok_code(cls):
        return cls.get_config_attribute("R4_fhir_coverage_eligibility_config").get('fhir_is_item_ok_code', 'is_item_ok')

    @classmethod
    def get_fhir_is_service_ok_code(cls):
        return cls.get_config_attribute("R4_fhir_coverage_eligibility_config").get('fhir_is_service_ok_code', 'is_service_ok')

    @classmethod
    def get_fhir_balance_code(cls):
        return cls.get_config_attribute("R4_fhir_coverage_eligibility_config").get('fhir_balance_code', 'balance')

    @classmethod
    def get_fhir_balance_default_category(cls):
        return cls.get_config_attribute("R4_fhir_coverage_eligibility_config").get('fhir_balance_default_category', 'medical')

    @classmethod
    def get_fhir_active_policy_status(cls):
        return cls.get_config_attribute("R4_fhir_coverage_eligibility_config").get('fhir_active_policy_status', ('A', 2))



import copy

from product.models import Product
from api_fhir_r4.converters import InsurancePlanConverter
from api_fhir_r4.exceptions import FHIRException
from api_fhir_r4.serializers import BaseFHIRSerializer


class InsurancePlanSerializer(BaseFHIRSerializer):
    fhirConverter = InsurancePlanConverter()

    def create(self, validated_data):
        code = validated_data.get('code')
        if Product.objects.filter(code=code).count() > 0:
            raise FHIRException('Exists product with following code `{}`'.format(code))
        copied_data = copy.deepcopy(validated_data)
        del copied_data['_state']
        # TODO services in product hasn't been developed yet.
        return Product.objects.create(**copied_data)

    def update(self, instance, validated_data):
        instance.code = validated_data.get('code', instance.code)
        instance.name = validated_data.get('name', instance.name)
        instance.date_from = validated_data.get('date_from', instance.date_from)
        instance.date_to = validated_data.get('date_to', instance.date_from)
        instance.location = validated_data.get('location', instance.location)
        instance.member_count = validated_data.get('member_count', instance.member_count)
        instance.insurance_period = validated_data.get('insurance_period', instance.insurance_period)
        instance.lump_sum = validated_data.get('lump_sum', instance.lump_sum)
        instance.threshold = validated_data.get('threshold', instance.threshold)
        instance.premium_adult = validated_data.get('premium_adult', instance.premium_adult)
        instance.premium_child = validated_data.get('premium_child', instance.premium_child)
        instance.registration_lump_sum = validated_data.get('registration_lump_sum', instance.registration_lump_sum)
        instance.registration_fee = validated_data.get('registration_fee', instance.registration_fee)
        instance.general_assembly_lump_sum = validated_data.get(
            'general_assembly_lump_sum', instance.general_assembly_lump_sum
        )
        instance.general_assembly_fee = validated_data.get('general_assembly_fee', instance.general_assembly_fee)
        instance.conversion_product = validated_data.get('conversion_product', instance.conversion_product)
        instance.max_installments = validated_data.get('max_installments', instance.max_installments)
        instance.grace_period_enrolment = validated_data.get('grace_period_enrolment', instance.grace_period_enrolment)
        instance.audit_user_id = self.get_audit_user_id()
        instance.save()
        return instance



from typing import List, Type, Dict

from fhir.resources.R4B import FHIRAbstractModel

from api_fhir_r4.containedResources.containedResourceHandler import ContainedResourceManager
from api_fhir_r4.containedResources.containedResources import AbstractContainedResourceCollection
from api_fhir_r4.serializers import BaseFHIRSerializer


class ContainedContentSerializerMixin:
    """
    Mixin for extending BaseFHIRSerializer. The creation of a FHIR representation through to_representation is extended
    with a "contained" value. It contains model attributes mapped to FHIR through ContainedResourceConverters
    listed contained_resources. The contained values are added only if the 'contained'
    value in the serializer context is set to True.
    """
    ALLOWED_RESOURCE_UPDATE_CONTAINED = {
        'Group':('Patient'), 
        'Patient':('Group'),
        'Claim':('Medication','ActivityDefinition','Organization','Patient','Practitioner','Group'),
        }

    #  Used for determining what reference type will be used used in contained value,
    # if None then value from ContainedResourceManager is used

    def __init__(self, *args, **kwargs):
        self.__contained_definitions = None
        # Serializers for contained resources are created with same argument as main serializer
        self.build_contained_resource_managers(*args, **kwargs)
        super().__init__(*args, **kwargs)

    @property
    def _contained_definitions(self) -> AbstractContainedResourceCollection:
        return self.__contained_definitions

    def build_contained_resource_managers(self, *args, **kwargs):
        self.__contained_definitions = self.contained_resources(*args, **kwargs)

    @property
    def contained_resources(self) -> Type[AbstractContainedResourceCollection]:
        """ Collection definition, used to determine which managers will be used for defining contained resources.
        :return:
        """
        raise NotImplementedError('Serializer with contained resources require contained_resources implemented')

    def fhir_object_reference_fields(self, fhir_obj: FHIRAbstractModel) -> List[FHIRAbstractModel]:
        """
        When contained resources are used, the references in fhir object fields should
        change to the contained resource reference starting with hash.
        References for values listed in this property will be changed.
        :return: List of fields from fhir_objects with references, which have representation in contained resources
        """
        raise NotImplementedError('fhir_object_reference_fields not implemented')

    def _get_converted_resources(self, obj):
        converted_values = []
        for resource in self._contained_definitions.get_contained().values():
            resource_fhir_repr = resource.convert_to_fhir(obj)
            converted_values.append((resource, resource_fhir_repr))
        return converted_values

    def to_internal_value(self, data):
        audit_user_id = self.get_audit_user_id()
        return self.fhirConverter.to_imis_obj(data, audit_user_id).__dict__

    def create(self, validated_data):
        self._create_or_update_contained(validated_data)
        super(ContainedContentSerializerMixin, self).create(validated_data)

    def update(self, instance, validated_data):
        self._create_or_update_contained(validated_data)
        super(ContainedContentSerializerMixin, self).update(instance, validated_data)

    def to_representation(self, obj):
        base_fhir_obj_repr = super(ContainedContentSerializerMixin, self).to_representation(obj)
        if self.context.get('contained', False):
            base_fhir_obj_repr['contained'] = self._create_contained_obj_dict(obj)
        return base_fhir_obj_repr

    def _create_contained_obj_dict(self, obj):
        contained_resources = self.create_contained_resource_fhir_implementation(obj)
        dict_list = [resource.dict() for resource in contained_resources]
        return dict_list

    def create_contained_resource_fhir_implementation(self, obj) -> List[FHIRAbstractModel]:
        contained_resources = []
        for resource, fhir_repr in self._get_converted_resources(obj):
            contained_resources.extend(fhir_repr)
        return contained_resources

    def _add_contained_references(self, fhir_obj: FHIRAbstractModel):
        for field in self.fhir_object_reference_fields(fhir_obj):
            field.reference = self._create_contained_reference(field.reference)

    def _create_contained_reference(self, base_reference):
        # Contained references are made by adding hash
        return F"#{base_reference}"

    def _create_or_update_contained(self, validated_data):
        result = {}
        main_resource_type = validated_data['resourceType']
        #TODO: use a bundle instead 
        if 'contained' in validated_data\
            and main_resource_type in self.ALLOWED_RESOURCE_UPDATE_CONTAINED:
            for resource in self._contained_definitions.get_contained().values():
                name = resource.alias
                ressource_type = resource.imis_converter.fhir_resource_type
                for contained in  validated_data['contained']:
                    if 'resourceType' in contained and contained['resourceType'] == ressource_type\
                        and contained['resourceType'] in self.ALLOWED_RESOURCE_UPDATE_CONTAINED[main_resource_type]:
                        result[name] = resource.create_or_update_from_contained(contained)
                        break
        return result



from rest_framework import status


class FhirApiReadTestMixin(object):

    @property
    def base_url(self):
        raise NotImplementedError()

    def login(self):
        raise NotImplementedError()

    def test_get_bad_authorization(self):
        response = self.client.get(self.base_url, data=None, format='json')
        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)

    def test_get_should_return_200(self):
        self.login()
        response = self.client.get(self.base_url, data=None, format='json')
        self.assertEqual(response.status_code, status.HTTP_200_OK)



from api_fhir_r4.tests import GenericTestMixin
from invoice.models import (
    Invoice,
    PaymentInvoice,
    DetailPaymentInvoice
)


class PaymentNoticeTestMixin(GenericTestMixin):
    _TEST_PAYMENT_NOTICE_IMIS_RECONCILIATION_STATUS = PaymentInvoice.ReconciliationStatus.RECONCILIATED
    _TEST_PAYMENT_NOTICE_IMIS_JSON_EXT = {'reconciliation': {'id': 'id-renconiliation-test-1'}}
    _TEST_PAYMENT_NOTICE_IMIS_DATE_PAYMENT = '2022-05-17'
    _TEST_PAYMENT_NOTICE_IMIS_DATE_CREATED = '2022-05-17T10:58:46.807554'
    _TEST_PAYMENT_NOTICE_IMIS_AMOUNT_RECEIVED = 10000.0
    _TEST_PAYMENT_NOTICE_IMIS_DETAILS_STATUS = DetailPaymentInvoice.DetailPaymentStatus.ACCEPTED
    _TEST_PAYMENT_NOTICE_IMIS_DETAILS_PAYMENT = None
    _TEST_PAYMENT_NOTICE_IMIS_DETAILS_SUBJECT_ID = 'bd84b2f2-ec1d-48de-8f8c-5a477aa4a29f'
    _TEST_PAYMENT_NOTICE_IMIS_DETAILS_RECON_ID = 'id-renconiliation-test-1'

    _TEST_PAYMENT_NOTICE_FHIR_STATUS = 'active'
    _TEST_PAYMENT_NOTICE_FHIR_REQUEST_REFERENCE = 'Invoice/bd84b2f2-ec1d-48de-8f8c-5a477aa4a29f'
    _TEST_PAYMENT_NOTICE_FHIR_REQUEST_REFERENCE_TYPE = 'Invoice'
    _TEST_PAYMENT_NOTICE_FHIR_CREATED = '2022-05-17T10:58:46.807554'
    _TEST_PAYMENT_NOTICE_FHIR_DATE_PAYMENT = '2022-05-17'
    _TEST_PAYMENT_NOTICE_FHIR_PAYMENT_REFERENCE = 'PaymentReconciliation/id-renconiliation-test-1'
    _TEST_PAYMENT_NOTICE_FHIR_RECIPIENT = 'Organization/openIMIS-Implementation'

    _TEST_PAYMENT_NOTICE_FHIR_AMOUNT_VALUE = '10000.0'
    _TEST_PAYMENT_NOTICE_FHIR_AMOUNT_CURRENCY = '$'

    _TEST_PAYMENT_NOTICE_FHIR_PAYMENT_STATUS = 'cleared'

    def create_test_imis_instance(self):
        imis_payment = PaymentInvoice(
            **{
                'reconciliation_status': self._TEST_PAYMENT_NOTICE_IMIS_RECONCILIATION_STATUS,
                'amount_received': self._TEST_PAYMENT_NOTICE_IMIS_AMOUNT_RECEIVED,
                'date_payment': self._TEST_PAYMENT_NOTICE_IMIS_DATE_PAYMENT,
                'json_ext': self._TEST_PAYMENT_NOTICE_IMIS_JSON_EXT,
                'payer_ref': 'ICD-54546'
            }
        )
        self._TEST_PAYMENT_NOTICE_IMIS_DETAILS_PAYMENT = imis_payment
        imis_payment_detail = DetailPaymentInvoice(
            **{
                'status': self._TEST_PAYMENT_NOTICE_IMIS_DETAILS_STATUS,
                'amount': self._TEST_PAYMENT_NOTICE_IMIS_AMOUNT_RECEIVED,
                'payment': self._TEST_PAYMENT_NOTICE_IMIS_DETAILS_PAYMENT,
                'subject_id': self._TEST_PAYMENT_NOTICE_IMIS_DETAILS_SUBJECT_ID
            }
        )
        return imis_payment, imis_payment_detail

    def verify_imis_instance(self, imis_obj):
        self.assertEquals(imis_obj.reconciliation_status, self._TEST_PAYMENT_NOTICE_IMIS_RECONCILIATION_STATUS)
        self.assertEquals(round(float(imis_obj.amount_received),2), round(float(self._TEST_PAYMENT_NOTICE_IMIS_AMOUNT_RECEIVED),2))
        self.assertEquals(f'{imis_obj.date_payment}', self._TEST_PAYMENT_NOTICE_IMIS_DATE_PAYMENT)
        self.assertEquals(imis_obj.json_ext['reconciliation']['id'], self._TEST_PAYMENT_NOTICE_IMIS_JSON_EXT['reconciliation']['id'])
        self.assertEquals(imis_obj.payer_ref, 'PaymentReconciliation/id-renconiliation-test-1')

    def verify_imis_detail_instance(self, imis_obj, imis_detail_obj):
        self.assertEquals(imis_detail_obj.status, self._TEST_PAYMENT_NOTICE_IMIS_DETAILS_STATUS)
        self.assertEquals(imis_detail_obj.amount, self._TEST_PAYMENT_NOTICE_IMIS_AMOUNT_RECEIVED)
        self.assertEquals(imis_detail_obj.payment.id, imis_obj.id)
        self.assertEquals(imis_detail_obj.subject_id, self._TEST_PAYMENT_NOTICE_IMIS_DETAILS_SUBJECT_ID)
        self.assertEquals(imis_detail_obj.reconcilation_id, self._TEST_PAYMENT_NOTICE_IMIS_DETAILS_RECON_ID)

    def verify_imis_invoice_status(self, imis_invoice_status):
        self.assertEquals(imis_invoice_status, Invoice.Status.PAID)

    def create_test_fhir_instance(self):
        return {
            'status': self._TEST_PAYMENT_NOTICE_FHIR_STATUS,
            'request': {
                'reference': self._TEST_PAYMENT_NOTICE_FHIR_PAYMENT_REFERENCE,
                'type': self._TEST_PAYMENT_NOTICE_FHIR_REQUEST_REFERENCE_TYPE
            },
            'created': self._TEST_PAYMENT_NOTICE_FHIR_CREATED,
            'payment': {
                'reference': self._TEST_PAYMENT_NOTICE_FHIR_PAYMENT_REFERENCE
            },
            'paymentDate': self._TEST_PAYMENT_NOTICE_IMIS_DATE_PAYMENT,
            'recipient': {
                'reference': self._TEST_PAYMENT_NOTICE_FHIR_RECIPIENT
            },
            'amount': {
                'value': self._TEST_PAYMENT_NOTICE_FHIR_AMOUNT_VALUE,
                'currency': "$"
            },
            "paymentStatus": {
                "coding": [
                    {
                        "system": "http://terminology.hl7.org/CodeSystem/paymentstatus",
                        "code": self._TEST_PAYMENT_NOTICE_FHIR_PAYMENT_STATUS,
                        "display": self._TEST_PAYMENT_NOTICE_FHIR_PAYMENT_STATUS
                    }
                ]
            }
        }

    def verify_fhir_instance(self, fhir_obj):
        self.assertEquals(fhir_obj.status, self._TEST_PAYMENT_NOTICE_FHIR_STATUS)
        self.assertEquals(fhir_obj.request.reference, self._TEST_PAYMENT_NOTICE_FHIR_REQUEST_REFERENCE)
        self.assertEquals(f'{fhir_obj.paymentDate}', self._TEST_PAYMENT_NOTICE_FHIR_DATE_PAYMENT)
        self.assertEquals(fhir_obj.recipient.reference, self._TEST_PAYMENT_NOTICE_FHIR_RECIPIENT)
        self.assertEquals(round(float(fhir_obj.amount.value),2), round(float(self._TEST_PAYMENT_NOTICE_FHIR_AMOUNT_VALUE),2))
        self.assertEquals(fhir_obj.paymentStatus.coding[0].code, self._TEST_PAYMENT_NOTICE_FHIR_PAYMENT_STATUS)



from medical.models import Service

from api_fhir_r4.configurations import GeneralConfiguration


class ServiceTypeMapping(object):
    SYSTEM = f"{GeneralConfiguration.get_system_base_url()}CodeSystem/activity-definition-service-type.html"

    fhir_service_type_coding = {
        Service.TYPE_PREVENTATIVE: {
            "system": SYSTEM,
            "code": "P",
            "display": "Preventive",
        },
        Service.TYPE_CURATIVE: {
            "system": SYSTEM,
            "code": "C",
            "display": "Curative",
        },
    }


class ServiceLevelMapping(object):
    SYSTEM = f"{GeneralConfiguration.get_system_base_url()}ValueSet/activity-definition-level"

    fhir_service_level_coding = {
        Service.LEVEL_SIMPLE_SERVICE: {
            "system": SYSTEM,
            "code": "S",
            "display": "Simple Service",
        },
        Service.LEVEL_VISIT: {
            "system": SYSTEM,
            "code": "V",
            "display": "Visit",
        },
        Service.LEVEL_DAY_HOSPITAL: {
            "system": SYSTEM,
            "code": "D",
            "display": "Day of stay",
        },
        Service.LEVEL_HOSPITAL_CARE: {
            "system": SYSTEM,
            "code": "H",
            "display": "Hospital case",
        },
    }


class UseContextMapping(object):
    SYSTEM = "http://terminology.hl7.org/CodeSystem/usage-context-type"

    fhir_use_context_coding = {
        "gender": {
            "system": SYSTEM,
            "code": "gender",
            "display": "Gender",
        },
        "age": {
            "system": SYSTEM,
            "code": "age",
            "display": "Age Range",
        },
        "venue": {
            "system": SYSTEM,
            "code": "venue",
            "display": "Clinical Venue",
        },
        "workflow": {
            "system": SYSTEM,
            "code": "workflow",
            "display": "Workflow Setting",
        },
    }


class VenueMapping(object):
    SYSTEM = "http://terminology.hl7.org/2.1.0/CodeSystem-v3-ActCode.html"

    fhir_venue_coding = {
        Service.CARE_TYPE_OUT_PATIENT: {
            "system": SYSTEM,
            "code": "AMB",
            "display": "ambulatory",
        },
        Service.CARE_TYPE_IN_PATIENT: {
            "system": SYSTEM,
            "code": "IMP",
            "display": "IMP",
        }
    }

    imis_venue_coding = {
        "AMB": Service.CARE_TYPE_OUT_PATIENT,
        "IMP": Service.CARE_TYPE_IN_PATIENT
    }


class WorkflowMapping(object):
    SYSTEM = f"{GeneralConfiguration.get_system_base_url()}CodeSystem/activity-definition-usage-context-workflow-type"

    fhir_workflow_coding = {
        Service.CATEGORY_SURGERY: {
            "system": SYSTEM,
            "code": "S",
            "display": "Surgery",
        },
        Service.CATEGORY_CONSULTATION: {
            "system": SYSTEM,
            "code": "C",
            "display": "Consultations",
        },
        Service.CATEGORY_DELIVERY: {
            "system": SYSTEM,
            "code": "D",
            "display": "Delivery",
        },
        Service.CATEGORY_ANTENATAL: {
            "system": SYSTEM,
            "code": "A",
            "display": "Antenatal",
        },
        Service.CATEGORY_OTHER: {
            "system": SYSTEM,
            "code": "O",
            "display": "Other",
        },
    }



from rest_framework import viewsets

from api_fhir_r4.mixins import MultiIdentifierRetrieverMixin
from api_fhir_r4.model_retrievers import UUIDIdentifierModelRetriever, CodeIdentifierModelRetriever
from api_fhir_r4.permissions import FHIRApiCommunicationRequestPermissions
from api_fhir_r4.serializers import CommunicationSerializer
from api_fhir_r4.views.fhir.base import BaseFHIRView
from api_fhir_r4.views.filters import ValidityFromRequestParameterFilter
from claim.models import Feedback

import logging
logger = logging.getLogger(__name__)

class CommunicationViewSet(BaseFHIRView, MultiIdentifierRetrieverMixin, viewsets.ModelViewSet):
    retrievers = [UUIDIdentifierModelRetriever, CodeIdentifierModelRetriever]
    serializer_class = CommunicationSerializer
    permission_classes = (FHIRApiCommunicationRequestPermissions,)

    def list(self, request, *args, **kwargs):
        queryset = self.get_queryset()
        identifier = request.GET.get("identifier")
        if identifier:
            return self.retrieve(request, *args, **{**kwargs, 'identifier': identifier})
        else:
            queryset = queryset.filter(validity_to__isnull=True)
        serializer = CommunicationSerializer(self.paginate_queryset(queryset), many=True)
        return self.get_paginated_response(serializer.data)

    def retrieve(self, *args, **kwargs):
        response = super().retrieve(self, *args, **kwargs)
        return response

    def get_queryset(self):
        queryset = Feedback.objects.filter(validity_to=None).order_by('validity_from')
        return ValidityFromRequestParameterFilter(self.request).filter_queryset(queryset)



from core import datetime

from insuree.test_helpers import create_test_insuree
from location.models import HealthFacility
from location.test_helpers import create_test_village, create_test_health_facility
from medical.test_helpers import create_test_item, create_test_service
from api_fhir_r4.configurations import R4IdentifierConfig

from api_fhir_r4.tests import GenericTestMixin, LocationTestMixin
from api_fhir_r4.utils import TimeUtils
from claim.models import Claim, ClaimItem, ClaimService
from medical.models import Diagnosis
from claim.test_helpers import create_test_claim_admin


class ClaimResponseTestMixin(GenericTestMixin):
    _TEST_CODE = 'codeTest'
    _TEST_STATUS = Claim.STATUS_ENTERED
    _TEST_STATUS_DISPLAY = "entered"
    _TEST_OUTCOME = "queued"
    _TEST_ADJUSTMENT = "adjustment"
    _TEST_DATE_PROCESSED = "2010-11-16T00:00:00"
    _TEST_APPROVED = 1000.00
    _TEST_REJECTION_REASON = 0
    _TEST_VISIT_TYPE = "O"

    # claim item data
    _TEST_ITEM_CODE = "iCode"
    _TEST_ITEM_UUID = "e2bc1546-390b-4d41-8571-632ecf7a0936"
    _TEST_ITEM_STATUS = Claim.STATUS_ENTERED
    _TEST_ITEM_QUANTITY = 20
    _TEST_ITEM_PRICE = 10.0
    _TEST_ITEM_REJECTED_REASON = 0

    # claim service data
    _TEST_SERVICE_CODE = "sCode"
    _TEST_SERVICE_UUID = "a17602f4-e9ff-4f42-a6a4-ccefcb23b4d6"
    _TEST_SERVICE_STATUS = Claim.STATUS_ENTERED
    _TEST_SERVICE_QUANTITY = 1
    _TEST_SERVICE_PRICE = 800
    _TEST_SERVICE_REJECTED_REASON = 0

    _TEST_ID = 9999
    _PRICE_ASKED = 1000
    _PRICE_APPROVED = 1000
    _ADMIN_AUDIT_USER_ID = 1

    _TEST_CLAIM_UUID = "ae580700-0277-4c98-adab-d98c0f7e681b"
    _TEST_ITEM_AVAILABILITY = True

    _TEST_ITEM_TYPE = 'D'
    _TEST_SERVICE_TYPE = 'D'

    # insuree and claim admin data
    _TEST_INSUREE_UUID = "76aca309-f8cf-4890-8f2e-b416d78de00b"
    _TEST_CLAIM_ADMIN_UUID = "044c33d1-dbf3-4d6a-9924-3797b461e535"

    # hf test data
    _TEST_HF_ID = 10000
    _TEST_HF_UUID = "6d0eea8c-62eb-11ea-94d6-c36229a16c2f"
    _TEST_HF_CODE = "12345678"
    _TEST_HF_NAME = "TEST_NAME"
    _TEST_HF_LEVEL = "H"
    _TEST_HF_LEGAL_FORM = "G"
    _TEST_ADDRESS = "TEST_ADDRESS"
    _TEST_PHONE = "133-996-476"
    _TEST_FAX = "1-408-999 8888"
    _TEST_EMAIL = "TEST@TEST.com"
    test_insuree = None
    test_claim_admin = None
    test_hf = None
    test_claim = None
    sub_str={}

    def setUp(self):
        super(ClaimResponseTestMixin, self).setUp()
        self.test_insuree = create_test_insuree()
        self.create_test_hf()
        self.test_claim_admin = create_test_claim_admin(custom_props={'health_facility_id':self.test_hf.id})
        self.test_claim = self.create_test_claim()
        self.test_item = self.create_test_claim_item()
        self.test_service = self.create_test_claim_service()
        self.sub_str[self._TEST_INSUREE_UUID] = self.test_insuree.uuid
        self.sub_str[self._TEST_CLAIM_ADMIN_UUID] = self.test_claim_admin.uuid
        self.sub_str[self._TEST_HF_UUID] = self.test_hf.uuid
        self.sub_str[self._TEST_CLAIM_UUID] = self.test_claim.uuid
        

    def create_test_claim_item(self):
        item = ClaimItem()
        item.item = create_test_item(
            self._TEST_ITEM_TYPE,
            custom_props={"code": self._TEST_ITEM_CODE}
        )
        item.claim = self.test_claim
        item.status = self._TEST_ITEM_STATUS
        item.qty_approved = self._TEST_ITEM_QUANTITY
        item.qty_provided = self._TEST_ITEM_QUANTITY
        item.rejection_reason = self._TEST_ITEM_REJECTED_REASON
        item.availability = self._TEST_ITEM_AVAILABILITY
        item.price_asked = self._TEST_ITEM_PRICE
        item.price_approved = self._TEST_ITEM_PRICE
        item.audit_user_id = self._ADMIN_AUDIT_USER_ID
        item.save()
        return item

    def create_test_claim_service(self):
        service = ClaimService()
        service.service = create_test_service(
            self._TEST_SERVICE_TYPE,
            custom_props={"code": self._TEST_SERVICE_CODE}
        )
        service.claim = self.test_claim
        service.status = self._TEST_SERVICE_STATUS
        service.qty_approved = self._TEST_SERVICE_QUANTITY
        service.qty_provided = self._TEST_SERVICE_QUANTITY
        service.rejection_reason = self._TEST_SERVICE_REJECTED_REASON
        service.availability = self._TEST_ITEM_AVAILABILITY
        service.price_asked = self._TEST_SERVICE_PRICE
        service.price_approved = self._TEST_SERVICE_PRICE
        service.audit_user_id = self._ADMIN_AUDIT_USER_ID
        service.save()
        return service

    def create_test_imis_instance(self):
        return self.test_claim
        
    def create_test_claim(self):
        imis_claim = Claim()
        imis_claim.id = self._TEST_ID
        imis_claim.uuid = self._TEST_CLAIM_UUID
        imis_claim.code = self._TEST_CODE
        imis_claim.status = self._TEST_STATUS
        imis_claim.adjustment = self._TEST_ADJUSTMENT
        imis_claim.date_processed = TimeUtils.str_to_date(self._TEST_DATE_PROCESSED)
        imis_claim.approved = self._TEST_APPROVED
        imis_claim.rejection_reason = self._TEST_REJECTION_REASON            
        imis_claim.insuree = self.test_insuree 
        imis_claim.health_facility = self.test_hf
        imis_claim.icd = Diagnosis(code='ICD00I', name='Test ICD')
        imis_claim.icd.audit_user_id = self._ADMIN_AUDIT_USER_ID
        imis_claim.icd.save()
        imis_claim.audit_user_id = self._ADMIN_AUDIT_USER_ID
        imis_claim.icd.date_from = datetime.date(2018, 12, 12)
        imis_claim.date_from = datetime.date(2018, 12, 12)
        imis_claim.date_claimed = datetime.date(2018, 12, 14)
        imis_claim.visit_type = self._TEST_VISIT_TYPE
            
        imis_claim.admin = self.test_claim_admin
        imis_claim.save()
        return imis_claim
    def create_test_hf(self):
        
        self.test_hf = create_test_health_facility(
            self._TEST_HF_CODE,
            self.test_insuree.family.location.parent.parent.id,
            custom_props = {
                'name': self._TEST_HF_NAME,
                'level':self._TEST_HF_LEVEL,
                'legal_form_id':self._TEST_HF_LEGAL_FORM,
                'address':self._TEST_ADDRESS,
                'phone':self._TEST_PHONE,
                'fax':self._TEST_FAX,
                'email':self._TEST_EMAIL,
            }
        )
        self._TEST_HF_ID = self.test_hf.id
        self._TEST_HF_UUID = self.test_hf.uuid
        return self.test_hf

    def verify_fhir_instance(self, fhir_obj):
        for identifier in fhir_obj.identifier:
            if identifier.type.coding[0].code == R4IdentifierConfig.get_fhir_uuid_type_code():
                self.assertEqual(str(self._TEST_CLAIM_UUID), identifier.value)
            elif identifier.type.coding[0].code == R4IdentifierConfig.get_fhir_claim_code_type():
                self.assertEqual(self._TEST_CODE, identifier.value)
        self.assertEqual(self._TEST_VISIT_TYPE, fhir_obj.type.coding[0].code)
        self.assertEqual(str(self._TEST_ITEM_STATUS), fhir_obj.item[0].adjudication[0].category.coding[0].code)
        self.assertEqual(self._TEST_ITEM_QUANTITY, fhir_obj.item[0].adjudication[0].value)
        self.assertEqual(self._TEST_ITEM_PRICE, fhir_obj.item[0].adjudication[0].amount.value)
        self.assertEqual(str(self._TEST_ITEM_REJECTED_REASON), fhir_obj.item[0].adjudication[0].reason.coding[0].code)
        self.assertEqual(str(self._TEST_SERVICE_STATUS), fhir_obj.item[1].adjudication[0].category.coding[0].code)
        self.assertEqual(self._TEST_SERVICE_QUANTITY, fhir_obj.item[1].adjudication[0].value)
        self.assertEqual(self._TEST_SERVICE_PRICE, fhir_obj.item[1].adjudication[0].amount.value)
        self.assertEqual(str(self._TEST_SERVICE_REJECTED_REASON),
                         fhir_obj.item[1].adjudication[0].reason.coding[0].code)
        self.assertEqual(str(self._TEST_STATUS), fhir_obj.total[0].category.coding[0].code)
        self.assertEqual(self._PRICE_ASKED, fhir_obj.total[0].amount.value)
        self.assertEqual(str(self.test_claim_admin.uuid), fhir_obj.requestor.identifier.value)
        self.assertEqual(str(self.test_insuree.uuid), fhir_obj.patient.identifier.value)



import json
import os

from django.db.models import Model
from rest_framework.test import APITestCase
from rest_framework import status
from typing import Type

from core.models import User,filter_validity
from core.services import create_or_update_interactive_user, create_or_update_core_user
from api_fhir_r4.tests import GenericFhirAPITestMixin
from api_fhir_r4.configurations import GeneralConfiguration
from api_fhir_r4.tests import LocationTestMixin, ClaimAdminPractitionerTestMixin
from api_fhir_r4.utils import DbManagerUtils, TimeUtils
from claim.models import Claim, ClaimAdmin
from location.models import HealthFacility, UserDistrict
from medical.models import Diagnosis, Item, Service
from location.test_helpers import create_test_location, create_test_health_facility, create_test_village
from api_fhir_r4.tests.utils import load_and_replace_json,get_connection_payload,get_or_create_user_api
from medical.test_helpers import create_test_item, create_test_service
from insuree.test_helpers import create_test_insuree
from claim.test_helpers import create_test_claim_admin

from insuree.models import Insuree, Family


class ClaimAPIContainedTestBaseMixin:
    base_url = GeneralConfiguration.get_base_url() + 'Claim/'
    _test_json_request_path = "/test/test_claim_contained.json"

    # diagnosis data
    _TEST_MAIN_ICD_CODE = 'T_CD'
    _TEST_MAIN_ICD_NAME = 'Test diagnosis'

    _TEST_CLAIM_ADMIN_UUID = "044c33d1-dbf3-4d6a-9924-3797b461e535"
    _TEST_INSUREE_UUID = "AAAA08B5-6E85-470C-83EC-0EE9370F0000"
    _TEST_INSUREE_CODE = "105000003"
    _TEST_GROUP_UUID = 'AAAA5232-9054-4D86-B4F2-0E9C4ADF0000'

    # claim item data
    _TEST_ITEM_CODE = "iCode"
    _TEST_ITEM_UUID = "e2bc1546-390b-4d41-8571-632ecf7a0936"
    _TEST_ITEM_QUANTITY_PROVIDED = 10.0
    _TEST_ITEM_PRICE_ASKED = 10.0
    _TEST_ITEM_EXPLANATION = "item_explanation"
    _TEST_ITEM_TYPE = 'D'

    # claim service data
    _TEST_SERVICE_CODE = "sCode"
    _TEST_SERVICE_UUID = "a17602f4-e9ff-4f42-a6a4-ccefcb23b4d6"
    _TEST_SERVICE_QUANTITY_PROVIDED = 1
    _TEST_SERVICE_PRICE_ASKED = 21000.0
    _TEST_SERVICE_EXPLANATION = "service_explanation"
    _TEST_SERVICE_TYPE = 'D'

    # hf test data
    _TEST_HF_ID = 10000
    _TEST_HF_UUID = 'AAAA5F9B-97C6-444B-AAD9-2FCCFD460000'
    _TEST_HF_CODE = "12345678"
    _TEST_HF_NAME = "TEST_NAME"
    _TEST_HF_LEVEL = "H"
    _TEST_HF_LEGAL_FORM = "G"
    _TEST_ADDRESS = "TEST_ADDRESS"
    _TEST_PHONE = "133-996-476"
    _TEST_FAX = "1-408-999 8888"
    _TEST_EMAIL = "TEST@TEST.com"

    _ADMIN_AUDIT_USER_ID = -1

    _test_json_path_credentials = "/test/test_login.json"
    _TEST_USER_NAME = "TestUserTest2"
    _TEST_USER_PASSWORD = "TestPasswordTest2"
    _TEST_DATA_USER = {
        "username": _TEST_USER_NAME,
        "last_name": _TEST_USER_NAME,
        "password": _TEST_USER_PASSWORD,
        "other_names": _TEST_USER_NAME,
        "user_types": "INTERACTIVE",
        "language": "en",
        "roles": [9],
    }
    _test_request_data_credentials = None
    test_village = None
    test_hf = None
    test_claim_admin = None
    test_insuree = None
    sub_str = {}

    def setUp(self):
        super(ClaimAPIContainedTestBaseMixin, self).setUp()
        self.create_dependencies()
        self.sub_str[self._TEST_GROUP_UUID] = self.test_insuree.family.uuid
        self.sub_str[self._TEST_INSUREE_UUID] = self.test_insuree.uuid
        self.sub_str[self._TEST_INSUREE_CODE] = self.test_insuree.chf_id
        self.sub_str[self._TEST_CLAIM_ADMIN_UUID] = self.test_claim_admin.uuid
        self.sub_str[self._TEST_HF_UUID] = self.test_hf.uuid
        self.sub_str[self._TEST_HF_CODE] = self.test_hf.code
        
        self._test_request_data = load_and_replace_json(self._test_json_request_path,self.sub_str)
        
        


    def create_dependencies(self):
        self._TEST_USER = get_or_create_user_api(self._TEST_DATA_USER)
        if not  self.test_insuree:
            self.test_insuree = create_test_insuree()
        UserDistrict.objects.create(
            **{
                'user':self._TEST_USER._u,
                'location': self.test_insuree.family.location.parent.parent,
                'audit_user_id':-1
            }
        )
        self.test_icd = Diagnosis()
        self.test_icd.code = self._TEST_MAIN_ICD_CODE
        self.test_icd.name = self._TEST_MAIN_ICD_NAME
        self.test_icd.audit_user_id = self._ADMIN_AUDIT_USER_ID
        self.test_icd.save()


        self.test_village = self.test_insuree.current_village or self.test_insuree.family.location
        self.test_hf=self.create_test_hf()
        self._TEST_HF_ID = self.test_hf.id
        if not self.test_claim_admin:
            self.test_claim_admin =create_test_claim_admin(
                custom_props={'health_facility_id': self.test_hf.id,
                                'last_name' : self._TEST_DATA_USER['last_name'],
                                'other_names' : self._TEST_DATA_USER['other_names']})
        

        ud = UserDistrict()
        ud.location = self.test_village.parent.parent
        ud.audit_user_id = self._ADMIN_AUDIT_USER_ID
        ud.user = self._TEST_USER.i_user
        ud.validity_from = TimeUtils.now()
        ud.save()
        
        self._TEST_USER.claim_admin = self.test_claim_admin
        self._TEST_USER.save()
  

    def create_test_claim_item(self):
        item = create_test_item(
            self._TEST_ITEM_TYPE,
            custom_props={"code": self._TEST_ITEM_CODE}
        )
        item.code = self._TEST_ITEM_CODE
        item.save()
        return item

    def create_test_claim_service(self):
        service = create_test_service(
            self._TEST_SERVICE_TYPE,
            custom_props={"code": self._TEST_SERVICE_CODE}
        )
        service.code = self._TEST_SERVICE_CODE
        service.save()
        return service

    def create_test_hf(self):
        hf = create_test_health_facility(
            self._TEST_HF_CODE,
            self.test_village.parent.parent.id,
            custom_props = {
                'name': self._TEST_HF_NAME,
                'level':self._TEST_HF_LEVEL,
                'legal_form_id':self._TEST_HF_LEGAL_FORM,
                'address':self._TEST_ADDRESS,
                'phone':self._TEST_PHONE,
                'fax':self._TEST_FAX,
                'email':self._TEST_EMAIL,
            }
        )
        return hf

    def assert_response(self, response_json):
        self.assertEqual(response_json["outcome"], 'complete')
        for item in response_json["item"]:
            for adjudication in item["adjudication"]:
                self.assertEqual(adjudication["category"]["coding"][0]["code"], f'{Claim.STATUS_REJECTED}')
                # 2 not in price list
                #FIXME familly in some cases self.assertEqual(adjudication["reason"]["coding"][0]["code"], '2')

        self.assertEqual(response_json["resourceType"], "ClaimResponse")

    def assert_hf_created(self):
        self._assert_unique_created(self.test_hf.uuid, HealthFacility)

    def assert_insuree_created(self):
        insuree = self._assert_unique_created(str(self.test_insuree.uuid), Insuree)

        family = self._assert_unique_created(str(self.test_insuree.family.uuid), Family)

        self.assertEqual(insuree, family.head_insuree)

    def assert_claim_admin_created(self):
        return None#FIXME
        expected_claim_admin_uuid = 'AAAA5229-DD11-4383-863C-E2FAD1B20000'
        self._assert_unique_created(expected_claim_admin_uuid, ClaimAdmin)
        # HF added using practitioner role
        admin = ClaimAdmin.objects.get(uuid=expected_claim_admin_uuid)
        hf = admin.health_facility
        self.assertEqual(hf.uuid, 'AAAA5F9B-97C6-444B-AAD9-2FCCFD460000')

    def assert_items_created(self):
        expected_item_uuid = 'AAAA76E2-DC28-4B48-8E29-3AC4ABEC0000'
        self._assert_unique_created(expected_item_uuid, Item)

        expected_service_uuid = 'AAAA29BA-3F4E-4E6F-B55C-23A488A10000'
        self._assert_unique_created(expected_service_uuid, Service)

    def _assert_unique_created(self, expected_uuid, django_model: Type[Model]):
        msg = f'Contained resource should create unique object of type ' \
              f'{django_model} with uuid {expected_uuid}'

        query = django_model.objects.filter(uuid=expected_uuid).all()

        self.assertEqual(query.count(), 1, msg)
        return query.get()

    def assert_hf_updated(self):
        return None#FIXME
        expected_updated_address = 'Uitly road 1'
        hf = HealthFacility.objects.get(uuid=self.test_hf.uuid)
        self.assertEqual(hf.address, expected_updated_address)

    def assert_contained(self, json_response):
        contained = json_response['contained']
        self.assertResourceExists(contained, 'Patient', 'AAAA08B5-6E85-470C-83EC-0EE9370F0000')
        self.assertResourceExists(contained, 'Organization', 'AAAA5F9B-97C6-444B-AAD9-2FCCFD460000')
        self.assertResourceExists(contained, 'Group', 'AAAA5232-9054-4D86-B4F2-0E9C4ADF0000')
        self.assertResourceExists(contained, 'Practitioner', 'AAAA5229-DD11-4383-863C-E2FAD1B20000')
        self.assertResourceExists(contained, 'Medication', 'AAAA76E2-DC28-4B48-8E29-3AC4ABEC0000')
        self.assertResourceExists(contained, 'ActivityDefinition', 'AAAA29BA-3F4E-4E6F-B55C-23A488A10000')

    def assertResourceExists(self, contained, resource_type, resource_id):
        x = [x for x in contained if x['resourceType'] == resource_type and x['id'] == resource_id]
        self.assertEqual(len(x), 1)


class ClaimAPIContainedTests(ClaimAPIContainedTestBaseMixin, GenericFhirAPITestMixin, APITestCase):
    base_url = GeneralConfiguration.get_base_url() + 'Claim/'
    _test_json_path = "/test/test_claim_contained.json"

    def test_post_should_create_correctly(self):
        response = self.client.post(
            GeneralConfiguration.get_base_url() + 'login/', data=get_connection_payload(self._TEST_DATA_USER), format='json'
        )
        response_json = response.json()
        token = response_json["token"]
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        headers = {
            "Content-Type": "application/json",
            "HTTP_AUTHORIZATION": f"Bearer {token}"
        }
        response = self.client.post(self.base_url, data=self._test_request_data, format='json', **headers)
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
        self.assertIsNotNone(response.content)
        response_json = response.json()
        self.assert_response(response_json)
        self.assert_hf_created()
        self.assert_insuree_created()
        self.assert_claim_admin_created()
        #self.assert_items_created()

    def test_post_should_update_contained_correctly(self):
        response = self.client.post(
            GeneralConfiguration.get_base_url() + 'login/', data=get_connection_payload(self._TEST_DATA_USER), format='json'
        )
        response_json = response.json()
        token = response_json["token"]
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        headers = {
            "Content-Type": "application/json",
            "HTTP_AUTHORIZATION": f"Bearer {token}"
        }


        # Confirm HF already exists
        self.assertTrue(HealthFacility.objects.filter(uuid=self.test_hf.uuid).exists())

        response = self.client.post(self.base_url, data=self._test_request_data, format='json', **headers)
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
        self.assertIsNotNone(response.content)
        response_json = response.json()
        self.assert_response(response_json)
        self.assert_hf_created()
        self.assert_insuree_created()
        self.assert_claim_admin_created()
        #self.assert_items_created()
        self.assert_hf_updated()

    def test_get_should_return_200_claim_with_contained(self):
        # Test if get Claim return contained resources
        response = self.client.post(
            GeneralConfiguration.get_base_url() + 'login/', data=get_connection_payload(self._TEST_DATA_USER), format='json'
        )
        response_json = response.json()
        token = response_json["token"]
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        headers = {
            "Content-Type": "application/json",
            "HTTP_AUTHORIZATION": f"Bearer {token}"
        }
        # Create claim
        response = self.client.post(self.base_url, data=self._test_request_data, format='json', **headers)
        response_json = response.json()
        self.assertTrue(status.is_success(response.status_code))
        uuid = response_json['id']
        url = F"{GeneralConfiguration.get_base_url()}Claim/{uuid}/?contained=True"
        response = self.client.get(url, data=None, format='json', **headers)
        #FIXME replace fix UUID with codes self.assert_contained(response.json())
        self.assertEqual(response.status_code, status.HTTP_200_OK)



from api_fhir_r4.models.fhirModels import (
    OperationOutcomeV2,
    UsageContextV2,
    CoverageV2,
    CoverageClassV2,
    ContractSignerV2,
    ClaimV2,
    ClaimInsuranceV2,
    ClaimResponseV2,
    CoverageEligibilityRequestV2,
)
from api_fhir_r4.models.imisModelEnums import BundleType
from api_fhir_r4.models.subscription import (
    Subscription,
    SubscriptionNotificationResult
)



from location.models import Location

from api_fhir_r4.configurations import R4IdentifierConfig, R4LocationConfig
from api_fhir_r4.converters import LocationConverter
from fhir.resources.R4B.location import Location as FHIRLocation
from api_fhir_r4.tests import GenericTestMixin
from location.test_helpers import create_test_location, create_test_health_facility, create_test_village


class LocationTestMixin(GenericTestMixin):

    _TEST_VILLAGE_CODE = "RTDTMTVT"
    _TEST_VILLAGE_NAME = "TEST_NAME"
    _TEST_LOCATION_TYPE = "V"
    test_region = None
    test_district = None
    test_village = None
    test_ward = None
    
    def setUp(self):
        super().setUp()
        if self.test_region is None:
            self.test_village  =create_test_village( custom_props={"code":self._TEST_VILLAGE_CODE,"name":self._TEST_VILLAGE_NAME})
            self.test_ward =self.test_village.parent
            self.test_region =self.test_village.parent.parent.parent
            self.test_district = self.test_village.parent.parent
    
    def create_test_imis_instance(self):
        return self.test_village

    def verify_imis_instance(self, imis_obj):
        self.assertEqual(self._TEST_VILLAGE_CODE, imis_obj.code)
        self.assertEqual(self._TEST_VILLAGE_NAME, imis_obj.name)
        self.assertEqual(self._TEST_LOCATION_TYPE, imis_obj.type)

    def create_test_fhir_instance(self):
        fhir_location = FHIRLocation.construct()
        identifier = LocationConverter.build_fhir_identifier(self._TEST_VILLAGE_CODE,
                                                             R4IdentifierConfig.get_fhir_identifier_type_system(),
                                                             R4IdentifierConfig.get_fhir_location_code_type())
        fhir_location.identifier = [identifier]

        fhir_location.name = self._TEST_VILLAGE_NAME

        system_definition = LocationConverter.PHYSICAL_TYPES.get(self._TEST_LOCATION_TYPE)
        fhir_location.physicalType = LocationConverter.build_codeable_concept(**system_definition)

        fhir_location.mode = 'instance'
        fhir_location.status = R4LocationConfig.get_fhir_code_for_active()

        return fhir_location

    def verify_fhir_instance(self, fhir_obj):
        for identifier in fhir_obj.identifier:
            code = LocationConverter.get_first_coding_from_codeable_concept(identifier.type).code
            if code == R4IdentifierConfig.get_fhir_uuid_type_code():
                self.assertEqual(fhir_obj.id, identifier.value)
            elif code == R4IdentifierConfig.get_fhir_facility_id_type():
                self.assertEqual(self._TEST_VILLAGE_CODE, identifier.value)
        self.assertEqual(self._TEST_VILLAGE_NAME, fhir_obj.name)
        physical_type_code = LocationConverter.get_first_coding_from_codeable_concept(fhir_obj.physicalType).code
        self.assertEqual(self._TEST_LOCATION_TYPE, physical_type_code)
        self.assertEqual(R4LocationConfig.get_fhir_code_for_active(), fhir_obj.status)
        self.assertEqual('instance', fhir_obj.mode)



from api_fhir_r4.converters import InsuranceOrganisationConverter
from api_fhir_r4.serializers import BaseFHIRSerializer


class InsuranceOrganizationSerializer(BaseFHIRSerializer):
    fhirConverter = InsuranceOrganisationConverter()

    def create(self, validated_data):
        pass

    def update(self, instance, validated_data):
        pass



from django.utils.translation import gettext as _
from api_fhir_r4.configurations import GeneralConfiguration, R4CoverageConfig
from api_fhir_r4.converters import BaseFHIRConverter, ReferenceConverterMixin
from api_fhir_r4.mapping.coverageMapping import CoverageStatus
from api_fhir_r4.models import CoverageV2 as Coverage, CoverageClassV2 as CoverageClass
from fhir.resources.R4B.period import Period
from fhir.resources.R4B.extension import Extension
from fhir.resources.R4B.reference import Reference
from policy.signals import signal_check_formal_sector_for_policy
from product.models import ProductItem, ProductService
from policy.models import Policy
from api_fhir_r4.utils import TimeUtils


class CoverageConverter(BaseFHIRConverter, ReferenceConverterMixin):

    @classmethod
    def to_fhir_obj(cls, imis_policy, reference_type=ReferenceConverterMixin.UUID_REFERENCE_TYPE):
        fhir_coverage = Coverage.construct()
        cls.build_coverage_status(fhir_coverage, imis_policy)
        cls.build_coverage_identifier(fhir_coverage, imis_policy)
        cls.build_coverage_policy_holder(fhir_coverage, imis_policy)
        cls.build_coverage_period(fhir_coverage, imis_policy)
        cls.build_coverage_class(fhir_coverage, imis_policy)
        cls.build_coverage_beneficiary(fhir_coverage, imis_policy)
        cls.build_coverage_payor(fhir_coverage, imis_policy)
        cls.build_coverage_extension(fhir_coverage, imis_policy)
        return fhir_coverage
 
    @classmethod
    def get_reference_obj_uuid(cls, imis_policy: Policy):
        return imis_policy.uuid

    @classmethod
    def get_reference_obj_id(cls, imis_policy: Policy):
        return imis_policy.id

    @classmethod
    def get_reference_obj_code(cls, imis_policy: Policy):
        # Policy doesn't have code representation, uuid is used instead
        return imis_policy.uuid

    @classmethod
    def build_fhir_code_identifier(cls, identifiers, imis_object):
        # Policy doesn't have code representation, uuid is used instead
        cls.build_fhir_uuid_identifier(identifiers, imis_object)

    @classmethod
    def get_fhir_resource_type(cls):
        return Coverage

    @classmethod
    def build_coverage_identifier(cls, fhir_coverage, imis_policy):
        identifiers = []
        cls.build_all_identifiers(identifiers, imis_policy)
        fhir_coverage.identifier = identifiers
        return fhir_coverage

    @classmethod
    def build_all_identifiers(cls, identifiers, imis_object):
        # Coverage don't have code
        cls.build_fhir_uuid_identifier(identifiers, imis_object)
        cls.build_fhir_id_identifier(identifiers, imis_object)
        return identifiers

    @classmethod
    def build_coverage_policy_holder(cls, fhir_coverage, imis_policy):
        reference = Reference.construct()
        resource_id = imis_policy.family.head_insuree.chf_id
        reference.reference = f'Patient/{str(resource_id)}'
        fhir_coverage.policyHolder = reference
        return fhir_coverage

    @classmethod
    def build_coverage_beneficiary(cls, fhir_coverage, imis_policy):
        reference = Reference.construct()
        resource_id = imis_policy.family.head_insuree.chf_id
        reference.reference = f'Patient/{str(resource_id)}'
        fhir_coverage.beneficiary = reference
        return fhir_coverage

    @classmethod
    def build_coverage_payor(cls, fhir_coverage, imis_policy):
        policy_holder_contract = None
        # send the signal from policy module - check if policy is connected to
        # formal sector contract entity
        results_signal_policy_fs = signal_check_formal_sector_for_policy.send(
             sender=cls, policy_id=imis_policy.id
        )
        if len(results_signal_policy_fs) > 0:
            if results_signal_policy_fs[0][1]:
                policy_holder_contract = results_signal_policy_fs[0][1]
        if policy_holder_contract:
            # formal sector
            resource_id = policy_holder_contract.id
            resource_type = 'Organization'
        else:
            # informal sector
            resource_id = imis_policy.family.head_insuree.chf_id
            resource_type = 'Patient'

        fhir_coverage.payor = []
        reference = Reference.construct()
        reference.reference = f'{resource_type}/{resource_id}'
        fhir_coverage.payor.append(reference)
        return fhir_coverage

    @classmethod
    def build_coverage_period(cls, fhir_coverage, imis_policy):
        period = Period.construct()
        if imis_policy.start_date is not None:
            period.start = imis_policy.start_date.isoformat()
        if imis_policy.expiry_date is not None:
            period.end = imis_policy.expiry_date.isoformat()
        fhir_coverage.period = period
        return period

    @classmethod
    def build_coverage_status(cls, fhir_coverage, imis_policy):
        code = imis_policy.status
        fhir_coverage.status = CoverageStatus.map_status(code)
        return fhir_coverage

    @classmethod
    def build_coverage_class(cls, fhir_coverage, imis_coverage):
        fhir_coverage.class_fhir = []

        coverage_class = CoverageClass.construct()
        product = imis_coverage.product
        coverage_class.value = product.code
        coverage_class.name = product.name
        coverage_class.type = cls.build_codeable_concept(
            system='http://terminology.hl7.org/CodeSystem/coverage-class',
            code='plan',
            display=_('Plan')
        )

        fhir_coverage.class_fhir.append(coverage_class)

    @classmethod
    def build_coverage_extension(cls, fhir_coverage, imis_coverage):
        cls.__build_enroll_date(fhir_coverage, imis_coverage)
        cls.__build_effective_date(fhir_coverage, imis_coverage)
        return fhir_coverage

    @classmethod
    def __build_effective_date(cls, fhir_coverage, imis_coverage):
        enroll_date = cls.__build_date_extension(imis_coverage.effective_date)
        if type(fhir_coverage.extension) is not list:
            fhir_coverage.extension = [enroll_date]
        else:
            fhir_coverage.extension.append(enroll_date)

    @classmethod
    def __build_enroll_date(cls, fhir_coverage, imis_coverage):
        enroll_date = cls.__build_date_extension(imis_coverage.enroll_date)
        if type(fhir_coverage.extension) is not list:
            fhir_coverage.extension = [enroll_date]
        else:
            fhir_coverage.extension.append(enroll_date)

    @classmethod
    def __build_date_extension(cls, value):
        ext_date = Extension.construct()
        ext_date.url = f'{GeneralConfiguration.get_system_base_url()}/StructureDefinition/coverage-date'
        ext_date.valueDate = TimeUtils.str_to_date(value.isoformat())
        return ext_date

    @classmethod
    def __build_product_plan_display(cls, class_, product):
        product_coverage = {}
        service_code = R4CoverageConfig.get_service_code()
        item_code = R4CoverageConfig.get_item_code()
        product_items = ProductItem.objects.filter(product=product).all()
        product_services = ProductService.objects.filter(product=product).all()
        product_coverage[item_code] = [item.item.code for item in product_items]
        product_coverage[service_code] = [service.service.code for service in product_services]
        class_.value = product.code
        class_.type = cls.build_simple_codeable_concept(product.name)
        class_.name = str(product_coverage)



from api_fhir_r4.configurations import R4IdentifierConfig
from api_fhir_r4.converters import ClaimAdminPractitionerRoleConverter
from fhir.resources.R4B.identifier import Identifier
from fhir.resources.R4B.practitionerrole import PractitionerRole
from fhir.resources.R4B.reference import Reference
from api_fhir_r4.tests import GenericTestMixin,  LocationTestMixin
from location.models import HealthFacility
from location.test_helpers import create_test_village, create_test_health_facility
from claim.test_helpers import create_test_claim_admin
from api_fhir_r4.utils import TimeUtils

class ClaimAdminPractitionerRoleTestMixin(GenericTestMixin):
    test_claim_admin = None
    test_hf = None
    test_village = None
    _TEST_ORGANIZATION_REFERENCE = None
    _TEST_CLAIM_ADMIN_PRACTITIONER_REFERENCE = None

    _TEST_CLAIM_ADMIN_ID = 1
    _TEST_CLAIM_ADMIN_UUID = "254f6268-964b-4d8d-aa26-20081f22235e"
    _TEST_CLAIM_ADMIN_CODE = "1234abcd"
    
    _TEST_CLAIM_ADMIN_DOB = "1990-03-24"

    _TEST_HF_ID = 10000
    _TEST_HF_UUID = "6d0eea8c-62eb-11ea-94d6-c36229a16c2f"
    _TEST_HF_CODE = "12345678"
    _TEST_HF_NAME = "TEST_NAME"
    _TEST_HF_LEVEL = "H"
    _TEST_HF_LEGAL_FORM = "G"
    _TEST_CLAIM_ADMIN_ADDRESS = "TEST_ADDRESS"
    _TEST_CLAIM_ADMIN_PHONE = "133-996-476"
    _TEST_CLAIM_ADMIN_FAX = "1-408-999 8888"
    _TEST_CLAIM_ADMIN_EMAIL = "TEST@TEST.com"
    sub_str = {}

    def setUp(self):
        super(ClaimAdminPractitionerRoleTestMixin, self).setUp()
        self.test_village= create_test_village()
        self.test_hf = self.create_test_hf()
        self.test_claim_admin = create_test_claim_admin( custom_props={
            'health_facility_id': self.test_hf.id, 
            'code':self._TEST_CLAIM_ADMIN_CODE,
            'dob':TimeUtils.str_to_date(self._TEST_CLAIM_ADMIN_DOB),
            'phone':self._TEST_CLAIM_ADMIN_PHONE})
        self._TEST_CLAIM_ADMIN_PRACTITIONER_REFERENCE = "Practitioner/" + str(self.test_claim_admin.uuid)
        self._TEST_ORGANIZATION_REFERENCE = "Organization/" + str(self.test_hf.uuid)
        self.sub_str[self._TEST_HF_UUID]=self.test_hf.uuid
        self.sub_str[self._TEST_CLAIM_ADMIN_UUID]=self.test_claim_admin.uuid
        self._TEST_HF_UUID=self.test_hf.uuid
        self._TEST_HF_ID=self.test_hf.id

    def create_test_hf(self):
        self.test_hf = create_test_health_facility(
            self._TEST_HF_CODE,
            self.test_village.parent.parent.id,
            custom_props = {
                'name': self._TEST_HF_NAME,
                'level':self._TEST_HF_LEVEL,
                'legal_form_id':self._TEST_HF_LEGAL_FORM,

            }
        )
        return self.test_hf


    def create_test_imis_instance(self):
        self.test_claim_admin.health_facility = self.test_hf
        return self.test_claim_admin

    def verify_imis_instance(self, imis_obj):
        self.assertEqual(self.test_hf.code, imis_obj.health_facility.code)

    def create_test_fhir_instance(self):
        fhir_practitioner_role = PractitionerRole.construct()
        identifiers = []
        code = ClaimAdminPractitionerRoleConverter.build_fhir_identifier(
            self._TEST_CLAIM_ADMIN_CODE,
            R4IdentifierConfig.get_fhir_identifier_type_system(),
            R4IdentifierConfig.get_fhir_generic_type_code()
        )
        identifiers.append(code)
        fhir_practitioner_role.identifier = identifiers
        organization_reference = Reference.construct()
        organization_reference.reference = self._TEST_ORGANIZATION_REFERENCE
        fhir_practitioner_role.organization = organization_reference
        practitioner_reference = Reference.construct()
        practitioner_reference.reference = self._TEST_CLAIM_ADMIN_PRACTITIONER_REFERENCE
        fhir_practitioner_role.practitioner = practitioner_reference
        return fhir_practitioner_role

    def verify_fhir_instance(self, fhir_obj):
        self.assertIn(self._TEST_ORGANIZATION_REFERENCE, fhir_obj.organization.reference)
        for identifier in fhir_obj.identifier:
            self.assertTrue(isinstance(identifier, Identifier))
            code = ClaimAdminPractitionerRoleConverter.get_first_coding_from_codeable_concept(identifier.type).code
            if code == R4IdentifierConfig.get_fhir_generic_type_code():
                self.assertEqual(self.test_claim_admin.code, identifier.value)
            elif code == R4IdentifierConfig.get_fhir_uuid_type_code():
                self.assertEqual(str(self.test_claim_admin.uuid), identifier.value)
        self.assertIn(self._TEST_CLAIM_ADMIN_PRACTITIONER_REFERENCE, fhir_obj.practitioner.reference)
        self.assertEqual(1, len(fhir_obj.code))
        self.assertEqual(1, len(fhir_obj.code[0].coding))
        self.assertEqual("CA", fhir_obj.code[0].coding[0].code)
        self.assertEqual("Claim Administrator", fhir_obj.code[0].coding[0].display)



from rest_framework import status
from rest_framework.response import Response
from rest_framework import exceptions, status, views


def call_default_exception_handler(exc, context):
    # Call REST framework's default exception handler first, to get the standard error response.
    response = views.exception_handler(exc, context)

    if isinstance(exc, (exceptions.AuthenticationFailed, exceptions.NotAuthenticated)):
        response.status_code = status.HTTP_401_UNAUTHORIZED
        return response
    return response


def fhir_api_exception_handler(exc, context):
    response = call_default_exception_handler(exc, context)

    request_path = __get_path_from_context(context)
    if 'api_fhir_r4' in request_path:
        from api_fhir_r4.converters import OperationOutcomeConverter
        fhir_outcome = OperationOutcomeConverter.to_fhir_obj(exc)
        if not response:
            response = __create_server_error_response()
        response.data = fhir_outcome.dict()

    return response


def __get_path_from_context(context):
    result = ""
    request = context.get("request")
    if request and request._request:
        result = request._request.path
    return result


def __create_server_error_response():
    return Response(None, status=status.HTTP_500_INTERNAL_SERVER_ERROR)



from api_fhir_r4.views.fhir.activity_definition import ActivityDefinitionViewSet
from api_fhir_r4.views.fhir.claim import ClaimViewSet
from api_fhir_r4.views.fhir.claim_response import ClaimResponseViewSet
from api_fhir_r4.views.fhir.code_systems.diagnosis import CodeSystemOpenIMISDiagnosisViewSet
from api_fhir_r4.views.fhir.code_systems.group_confirmation_type import CodeSystemOpenIMISGroupConfirmationTypeViewSet
from api_fhir_r4.views.fhir.code_systems.group_type import CodeSystemOpenIMISGroupTypeViewSet
from api_fhir_r4.views.fhir.code_systems.organization_hf_legal_form import CodeSystemOrganizationHFLegalFormViewSet
from api_fhir_r4.views.fhir.code_systems.organization_hf_level import CodeSystemOrganizationHFLevelViewSet
from api_fhir_r4.views.fhir.code_systems.organization_ph_activity import CodeSystemOrganizationPHActivityViewSet
from api_fhir_r4.views.fhir.code_systems.organization_ph_legal_form import CodeSystemOrganizationPHLegalFormViewSet
from api_fhir_r4.views.fhir.code_systems.patient_education_level import CodeSystemOpenIMISPatientEducationLevelViewSet
from api_fhir_r4.views.fhir.code_systems.patient_identification_type import \
    CodeSystemOpenIMISPatientIdentificationTypeViewSet
from api_fhir_r4.views.fhir.code_systems.patient_profession import CodeSystemOpenIMISPatientProfessionViewSet
from api_fhir_r4.views.fhir.code_systems.patient_relationship import CodeSystemOpenIMISPatientRelationshipViewSet
from api_fhir_r4.views.fhir.communication import CommunicationViewSet
from api_fhir_r4.views.fhir.communication_request import CommunicationRequestViewSet
from api_fhir_r4.views.fhir.contract import ContractViewSet
from api_fhir_r4.views.fhir.coverage_eligibility_request import CoverageEligibilityRequestViewSet
from api_fhir_r4.views.fhir.coverage_request import CoverageRequestQuerySet
from api_fhir_r4.views.fhir.group import GroupViewSet
from api_fhir_r4.views.fhir.insurance_plan import ProductViewSet
from api_fhir_r4.views.fhir.insuree import InsureeViewSet
from api_fhir_r4.views.fhir.invoice import InvoiceViewSet
from api_fhir_r4.views.fhir.location import LocationViewSet
from api_fhir_r4.views.fhir.medication import MedicationViewSet
from api_fhir_r4.views.fhir.organisation import OrganisationViewSet
from api_fhir_r4.views.fhir.practitioner import PractitionerViewSet
from api_fhir_r4.views.fhir.practitioner_role import PractitionerRoleViewSet
from api_fhir_r4.views.fhir.subscription import SubscriptionViewSet
from api_fhir_r4.views.fhir.payment_notice import PaymentNoticeViewSet



# Generated by Django 4.2.9 on 2024-02-07 16:47

import core.fields
import datetime
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django_cryptography.fields


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('api_fhir_r4', '0006_add_subsription_perms_imis_admin'),
    ]

    operations = [
        migrations.AlterField(
            model_name='historicalsubscription',
            name='criteria',
            field=models.JSONField(blank=True, db_column='Criteria', null=True),
        ),
        migrations.AlterField(
            model_name='historicalsubscription',
            name='date_created',
            field=core.fields.DateTimeField(db_column='DateCreated', default=datetime.datetime.now, null=True),
        ),
        migrations.AlterField(
            model_name='historicalsubscription',
            name='date_updated',
            field=core.fields.DateTimeField(db_column='DateUpdated', default=datetime.datetime.now, null=True),
        ),
        migrations.AlterField(
            model_name='historicalsubscription',
            name='headers',
            field=django_cryptography.fields.encrypt(models.TextField(blank=True, db_column='Headers', max_length=255, null=True)),
        ),
        migrations.AlterField(
            model_name='historicalsubscription',
            name='replacement_uuid',
            field=models.UUIDField(blank=True, db_column='ReplacementUUID', null=True),
        ),
        migrations.AlterField(
            model_name='subscription',
            name='criteria',
            field=models.JSONField(blank=True, db_column='Criteria', null=True),
        ),
        migrations.AlterField(
            model_name='subscription',
            name='date_created',
            field=core.fields.DateTimeField(db_column='DateCreated', default=datetime.datetime.now, null=True),
        ),
        migrations.AlterField(
            model_name='subscription',
            name='date_updated',
            field=core.fields.DateTimeField(db_column='DateUpdated', default=datetime.datetime.now, null=True),
        ),
        migrations.AlterField(
            model_name='subscription',
            name='headers',
            field=django_cryptography.fields.encrypt(models.TextField(blank=True, db_column='Headers', max_length=255, null=True)),
        ),
        migrations.AlterField(
            model_name='subscription',
            name='replacement_uuid',
            field=models.UUIDField(blank=True, db_column='ReplacementUUID', null=True),
        ),
        migrations.AlterField(
            model_name='subscription',
            name='user_created',
            field=models.ForeignKey(db_column='UserCreatedUUID', on_delete=django.db.models.deletion.DO_NOTHING, related_name='%(class)s_user_created', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterField(
            model_name='subscription',
            name='user_updated',
            field=models.ForeignKey(db_column='UserUpdatedUUID', on_delete=django.db.models.deletion.DO_NOTHING, related_name='%(class)s_user_updated', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterField(
            model_name='subscriptionnotificationresult',
            name='error',
            field=models.TextField(blank=True, default=None, null=True),
        ),
    ]



import json
import os

from fhir.resources.R4B.activitydefinition import ActivityDefinition

from api_fhir_r4.converters import ActivityDefinitionConverter
from api_fhir_r4.tests.mixin.activityDefinitionTestMixin import ActivityDefinitionTestMixin


class ActivityDefinitionConverterTestCase(ActivityDefinitionTestMixin):
    __TEST_ACTIVITY_DEFINITION_JSON_PATH = "/test/test_activity_definition.json"

    @classmethod
    def setUpClass(cls):
        super(ActivityDefinitionConverterTestCase, cls).setUpClass()
        dir_path = os.path.dirname(os.path.realpath(__file__))
        cls.__TEST_ACTIVITY_DEFINITION_JSON_TEXT__ = open(dir_path + cls.__TEST_ACTIVITY_DEFINITION_JSON_PATH).read()

    def setUp(self):
        super(ActivityDefinitionConverterTestCase, self).setUp()

    def test_to_fhir_obj(self):
        imis_service = self.create_test_imis_instance()
        fhir_activity_definition = ActivityDefinitionConverter.to_fhir_obj(imis_service)
        self.verify_fhir_instance(fhir_activity_definition)

    def test_to_imis_obj(self):
        fhir_activity_definition = self.create_test_fhir_instance()
        imis_service = ActivityDefinitionConverter.to_imis_obj(fhir_activity_definition.dict(), None)
        self.verify_imis_instance(imis_service)

    def test_create_object_from_json(self):
        dict_activity_definition = json.loads(
            ActivityDefinitionConverterTestCase.__TEST_ACTIVITY_DEFINITION_JSON_TEXT__)
        fhir_activity_definition = ActivityDefinition(**dict_activity_definition)
        self.verify_fhir_instance(fhir_activity_definition)



from api_fhir_r4.configurations import R4PaymentNoticeConfig
from invoice.models import PaymentInvoice


class PaymentNoticePaymentStatusMapping:
    _IMIS_RECONCILIATED_CLEARED = PaymentInvoice.ReconciliationStatus.RECONCILIATED.value
    _FHIR_RECONCILIATED_CLEARED = R4PaymentNoticeConfig.get_fhir_payment_notice_payment_status_cleared()

    _IMIS_NOT_RECONCILIATED_PAID = PaymentInvoice.ReconciliationStatus.NOT_RECONCILIATED.value
    _FHIR_NOT_RECONCILIATED_PAID = R4PaymentNoticeConfig.get_fhir_payment_notice_payment_status_paid()

    to_fhir_status = {
        _IMIS_RECONCILIATED_CLEARED: _FHIR_RECONCILIATED_CLEARED,
        _IMIS_NOT_RECONCILIATED_PAID: _FHIR_NOT_RECONCILIATED_PAID,
    }

    to_imis_status = {
        _FHIR_RECONCILIATED_CLEARED: _IMIS_RECONCILIATED_CLEARED,
        _FHIR_NOT_RECONCILIATED_PAID: _IMIS_NOT_RECONCILIATED_PAID,
    }



from rest_framework import status
from rest_framework.test import APITestCase
from api_fhir_r4.tests import GenericFhirAPITestMixin
from api_fhir_r4.configurations import GeneralConfiguration
from insuree.models import Profession


class CodeSystemPatientProfessionAPITests(GenericFhirAPITestMixin, APITestCase):
    base_url = GeneralConfiguration.get_base_url() + 'CodeSystem/patient-profession/'

    def setUp(self):
        super(CodeSystemPatientProfessionAPITests, self).setUp()
        self._EXPECTED_COUNT = Profession.objects.all().count()

    def test_get_bad_authorization(self):
        response = self.client.get(self.base_url, data=None, format='json')
        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)

    def test_get_should_return_code_system(self):
        self.login()
        response = self.client.get(self.base_url, data=None, format='json')
        response_data = response.json()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        employee = ""
        for concept in response_data['concept']:
            if concept['display'] == 'Employee':
                national_id = concept['display']
        self.assertEqual(national_id, 'Employee')
        self.assertEqual(response_data['count'], self._EXPECTED_COUNT)
        self.assertEqual(response_data['name'], 'PatientProfessionCS')
        self.assertEqual(response_data['title'], 'Profession (Patient)')



from abc import ABC
from typing import Union

from django.db.models import Model
from fhir.resources.R4B.extension import Extension
from fhir.resources.R4B.money import Money
from fhir.resources.R4B.quantity import Quantity

import core
from api_fhir_r4.configurations import R4IdentifierConfig
from api_fhir_r4.exceptions import FHIRRequestProcessException
from fhir.resources.R4B.codeableconcept import CodeableConcept
from fhir.resources.R4B.contactpoint import ContactPoint
from fhir.resources.R4B.address import Address
from fhir.resources.R4B.coding import Coding
from fhir.resources.R4B.reference import Reference
from fhir.resources.R4B.identifier import Identifier
from api_fhir_r4.configurations import GeneralConfiguration


class BaseFHIRConverter(ABC):

    @classmethod
    def to_fhir_obj(cls, obj, reference_type):
        raise NotImplementedError('`toFhirObj()` must be implemented.')  # pragma: no cover

    @classmethod
    def to_imis_obj(cls, data, audit_user_id):
        raise NotImplementedError('`toImisObj()` must be implemented.')  # pragma: no cover

    @classmethod
    def get_fhir_code_identifier_type(cls):
        raise NotImplementedError('get_fhir_code_identifier_type() must be implemented')

    @classmethod
    def _build_simple_pk(cls, fhir_obj, resource_id):
        if type(resource_id) is not str:
            resource_id = str(resource_id)
        fhir_obj.id = resource_id

    @classmethod
    def build_fhir_pk(cls, fhir_obj, resource: Union[str, Model], reference_type: str = None):
        if not reference_type:
            cls._build_simple_pk(fhir_obj, resource)
        if reference_type == ReferenceConverterMixin.UUID_REFERENCE_TYPE:
            # OE0-18 - change into string type uuid
            fhir_obj.id = str(resource.uuid)
        elif reference_type == ReferenceConverterMixin.DB_ID_REFERENCE_TYPE:
            fhir_obj.id = str(resource.id)
        elif reference_type == ReferenceConverterMixin.CODE_REFERENCE_TYPE:
            fhir_obj.id = resource.code

    @classmethod
    def valid_condition(cls, condition, error_message, errors=None):
        if errors is None:
            errors = []
        if condition:
            errors.append(error_message)
        return condition

    @classmethod
    def check_errors(cls, errors=None):  # pragma: no cover
        if errors is None:
            errors = []
        if len(errors) > 0:
            raise FHIRRequestProcessException(errors)

    @classmethod
    def build_simple_codeable_concept(cls, text):
        return cls.build_codeable_concept(None, None, text)

    @classmethod
    def build_codeable_concept(cls, code, system=None, text=None, display=None):
        codeable_concept = CodeableConcept.construct()
        if code or system:
            coding = Coding.construct()

            if GeneralConfiguration.show_system():
                coding.system = system

            coding.code = str(code)

            if display:
                coding.display = str(display)
            codeable_concept.coding = [coding]

        if text:
            codeable_concept.text = text
        return codeable_concept

    @classmethod
    def build_codeable_concept_from_coding(cls, coding, text=None):
        codeable_concept = CodeableConcept.construct()

        if coding:
            codeable_concept.coding = [coding]

        if text:
            codeable_concept.text = str(text)

        return codeable_concept

    @classmethod
    def get_first_coding_from_codeable_concept(cls, codeable_concept):
        result = Coding.construct()
        if codeable_concept:
            coding = codeable_concept.coding
            if coding and isinstance(coding, list) and len(coding) > 0:
                result = codeable_concept.coding[0]
        return result

    @classmethod
    def build_all_identifiers(cls, identifiers, imis_object, reference_type=None):
        cls.build_fhir_uuid_identifier(identifiers, imis_object)
        cls.build_fhir_code_identifier(identifiers, imis_object)
        if reference_type == ReferenceConverterMixin.DB_ID_REFERENCE_TYPE:
            cls.build_fhir_id_identifier(identifiers, imis_object)
        return identifiers

    @classmethod
    def build_fhir_uuid_identifier(cls, identifiers, imis_object):
        if hasattr(imis_object, 'uuid'):
            identifiers.append(cls.__build_uuid_identifier(imis_object.uuid))

    @classmethod
    def build_fhir_id_identifier(cls, identifiers, imis_object):
        if hasattr(imis_object, 'id'):
            identifiers.append(cls.__build_id_identifier(str(imis_object.id)))

    @classmethod
    def build_fhir_code_identifier(cls, identifiers, imis_object):
        if hasattr(imis_object, 'code'):
            identifiers.append(cls.__build_code_identifier(imis_object.code))

    @classmethod
    def __build_uuid_identifier(cls, uuid):
        return cls.build_fhir_identifier(uuid,
                                         R4IdentifierConfig.get_fhir_identifier_type_system(),
                                         R4IdentifierConfig.get_fhir_uuid_type_code())

    @classmethod
    def __build_id_identifier(cls, db_id):
        return cls.build_fhir_identifier(db_id,
                                         R4IdentifierConfig.get_fhir_identifier_type_system(),
                                         R4IdentifierConfig.get_fhir_id_type_code())

    @classmethod
    def __build_code_identifier(cls, code):
        return cls.build_fhir_identifier(code,
                                         R4IdentifierConfig.get_fhir_identifier_type_system(),
                                         cls.get_fhir_code_identifier_type())

    @classmethod
    def build_fhir_identifier(cls, value, type_system, type_code):
        identifier = Identifier.construct()
        type = cls.build_codeable_concept(type_code, type_system)
        identifier.type = type
        # OE0-18 - change into string type always
        identifier.value = str(value)
        return identifier

    @classmethod
    def get_fhir_identifier_by_code(cls, identifiers, lookup_code):
        value = None
        for identifier in identifiers or []:
            first_coding = cls.get_first_coding_from_codeable_concept(identifier.type)
            if first_coding.code == lookup_code:
                value = identifier.value
                break
        return value

    @classmethod
    def get_fhir_extension_by_url(cls, extensions, url):
        return next(iter([extension for extension in extensions if extension.url == url]), None)

    @classmethod
    def get_code_from_extension_codeable_concept(cls, extension):
        return extension.valueCodeableConcept.coding[0].code

    @classmethod
    def get_use_context_by_code(cls, use_context, code):
        return next(iter([entry for entry in use_context if entry.code.code == code]), None)

    @classmethod
    def build_fhir_contact_point(cls, value, system=None, use=None):
        contact_point = ContactPoint.construct()
        if system and GeneralConfiguration.show_system():
            contact_point.system = system.value

        if use:
            contact_point.use = use.value

        contact_point.value = value
        return contact_point

    @classmethod
    def get_contract_points_by_system_code(cls, telecom, system=None):
        return [contact.value for contact in telecom if contact.system == system]

    @classmethod
    def build_fhir_address(cls, value, use, type):
        current_address = Address.construct()
        if value:
            current_address.text = value
        current_address.use = use
        current_address.type = type
        return current_address

    @classmethod
    def build_fhir_reference(cls, identifier, display, ref_type, reference):
        fhir_reference = Reference.construct()
        fhir_reference.identifier = identifier
        fhir_reference.display = display
        fhir_reference.type = ref_type
        fhir_reference.reference = reference
        return fhir_reference

    @classmethod
    def build_fhir_mapped_coding(cls, mapping) -> Coding:
        coding = Coding.construct()

        if GeneralConfiguration.show_system():
            coding.system = mapping["system"]
        coding.code = mapping["code"]
        coding.display = mapping["display"]

        return coding

    @classmethod
    def build_fhir_reference_extension(cls, reference: Reference, url):
        extension = Extension.construct()
        extension.url = url
        extension.valueReference = reference
        return extension

    @classmethod
    def build_fhir_codeable_concept_extension(cls, codeable_concept: CodeableConcept, url):
        extension = Extension.construct()
        extension.url = url
        extension.valueCodeableConcept = codeable_concept
        return extension

    @classmethod
    def build_fhir_money(cls, value, currency=None):
        money = Money.construct()
        money.value = value
        if currency:
            money.currency = currency
        elif hasattr(core, 'currency'):
            money.currency = core.currency
        return money

    @classmethod
    def build_fhir_quantity(cls, value):
        quantity = Quantity.construct()
        quantity.value = value
        return quantity

    @classmethod
    def get_id_from_reference(cls, reference):
        splited_reference_string = reference.reference.split('/')
        id_from_reference = splited_reference_string.pop()
        return id_from_reference


from api_fhir_r4.converters.personConverterMixin import PersonConverterMixin
from api_fhir_r4.converters.referenceConverterMixin import ReferenceConverterMixin
from api_fhir_r4.converters.medicationConverter import MedicationConverter
from api_fhir_r4.converters.activityDefinitionConverter import ActivityDefinitionConverter
from api_fhir_r4.converters.contractConverter import ContractConverter
from api_fhir_r4.converters.patientConverter import PatientConverter
from api_fhir_r4.converters.groupConverter import GroupConverter
from api_fhir_r4.converters.policyHolderOrganisationConverter import PolicyHolderOrganisationConverter
from api_fhir_r4.converters.locationConverter import LocationConverter
from api_fhir_r4.converters.locationSiteConverter import LocationSiteConverter
from api_fhir_r4.converters.operationOutcomeConverter import OperationOutcomeConverter
from api_fhir_r4.converters.claimAdminPractitionerConverter import ClaimAdminPractitionerConverter
from api_fhir_r4.converters.claimAdminPractitionerRoleConverter import ClaimAdminPractitionerRoleConverter
from api_fhir_r4.converters.coverageEligibilityRequestConverter import CoverageEligibilityRequestConverter
# from api_fhir_r4.converters.policyCoverageEligibilityRequestConverter import PolicyCoverageEligibilityRequestConverter
from api_fhir_r4.converters.communicationRequestConverter import CommunicationRequestConverter
from api_fhir_r4.converters.claimResponseConverter import ClaimResponseConverter
from api_fhir_r4.converters.claimConverter import ClaimConverter
from api_fhir_r4.converters.insurancePlanConverter import InsurancePlanConverter
from api_fhir_r4.converters.codeSystemConverter import CodeSystemConverter
from api_fhir_r4.converters.healthFacilityOrganisationConverter import HealthFacilityOrganisationConverter
from api_fhir_r4.converters.insuranceOrganisationConverter import InsuranceOrganisationConverter
from api_fhir_r4.converters.enrolmentOfficerPractitionerConverter import EnrolmentOfficerPractitionerConverter
from api_fhir_r4.converters.enrolmentOfficerPractitionerRoleConverter import EnrolmentOfficerPractitionerRoleConverter
from api_fhir_r4.converters.communicationConverter import CommunicationConverter
from api_fhir_r4.converters.coverageConverter import CoverageConverter
from api_fhir_r4.converters.invoiceConverter import InvoiceConverter, BillInvoiceConverter



from api_fhir_r4.converters import ContractConverter
from api_fhir_r4.tests import ContractTestMixin
from fhir.resources.R4B.contract import Contract

from api_fhir_r4.tests.mixin import ConvertToImisTestMixin, ConvertToFhirTestMixin, ConvertJsonToFhirTestMixin


class ContractConverterTestCase(ContractTestMixin,
                                ConvertToImisTestMixin,
                                ConvertToFhirTestMixin,
                                ConvertJsonToFhirTestMixin):
    converter = ContractConverter
    fhir_resource = Contract
    json_repr = 'test/test_contract.json'



from api_fhir_r4.converters.contractConverter import ContractConverter
from api_fhir_r4.serializers import BaseFHIRSerializer
from contribution.services import update_or_create_premium
from policy.services import PolicyService
from policy.models import Policy
import copy
from api_fhir_r4.exceptions import FHIRException


class ContractSerializer(BaseFHIRSerializer):
    fhirConverter = ContractConverter

    def create(self, validated_data):
        request = self.context.get("request")
        user = request.user

        family = validated_data.get('family_id')
        insurees = validated_data.pop('insurees')
        premiums = validated_data.pop('contributions')

        if Policy.objects.filter(family_id=family).filter(start_date__range=[validated_data.get('effective_date'),validated_data.get('expiry_date')]).count() > 0:
            raise FHIRException('Contract exists for this patient')

        copied_data = copy.deepcopy(validated_data)
        del copied_data['_state']
        #TODO should we implement a way to create a resource with a given uuid
        del copied_data['uuid']

        new_policy = PolicyService(user).update_or_create(copied_data, user)
        # create contributions related to newly created policy
        if premiums:
            for premium in premiums:
                premium = update_or_create_premium(premium, user)
                new_policy = premium.policy

        return new_policy
    
    def update(self, instance, validated_data):
        instance.save()
        return instance



from api_fhir_r4.configurations import CommunicationRequestConfiguration


class R4CommunicationRequestConfig(CommunicationRequestConfiguration):

    @classmethod
    def build_configuration(cls, cfg):
        cls.get_config().R4_fhir_communication_config = cfg['R4_fhir_communication_request_config']

    @classmethod
    def get_fhir_care_rendered_code(cls):
        return cls.get_config_attribute("R4_fhir_communication_config").get('fhir_care_rendered_code', "CareRendered")

    @classmethod
    def get_fhir_payment_asked_code(cls):
        return cls.get_config_attribute("R4_fhir_communication_config").get('fhir_payment_asked_code', "PaymentAsked")

    @classmethod
    def get_fhir_drug_prescribed_code(cls):
        return cls.get_config_attribute("R4_fhir_communication_config").get('fhir_drug_prescribed_code', "DrugPrescribed")

    @classmethod
    def get_fhir_drug_received_code(cls):
        return cls.get_config_attribute("R4_fhir_communication_config").get('fhir_drug_received_code', "DrugReceived")

    @classmethod
    def get_fhir_asessment_code(cls):
        return cls.get_config_attribute("R4_fhir_communication_config").get('fhir_asessment_code', "Asessment")



from api_fhir_r4.configurations import MaritalConfiguration


class R4MaritalConfig(MaritalConfiguration):

    @classmethod
    def build_configuration(cls, cfg):
        cls.get_config().R4_fhir_marital_status_config = cfg['R4_fhir_marital_status_config']

    @classmethod
    def get_fhir_marital_status_system(cls):
        return cls.get_config_attribute("R4_fhir_marital_status_config").get('system',
                                                       "http://hl7.org/fhir/valueset-marital-status.html")

    @classmethod
    def get_fhir_married_code(cls):
        return cls.get_config_attribute("R4_fhir_marital_status_config").get('fhir_code_for_married', "M")

    @classmethod
    def get_fhir_never_married_code(cls):
        return cls.get_config_attribute("R4_fhir_marital_status_config").get('fhir_code_for_never_married', "S")

    @classmethod
    def get_fhir_divorced_code(cls):
        return cls.get_config_attribute("R4_fhir_marital_status_config").get('fhir_code_for_divorced', "D")

    @classmethod
    def get_fhir_widowed_code(cls):
        return cls.get_config_attribute("R4_fhir_marital_status_config").get('fhir_code_for_widowed', "W")

    @classmethod
    def get_fhir_unknown_marital_status_code(cls):
        return cls.get_config_attribute("R4_fhir_marital_status_config").get('fhir_code_for_unknown', "U")
